package com.yifan.yf_ads;

import android.app.Application;
import android.content.Context;
import android.content.Intent;

import androidx.annotation.NonNull;

import com.yifan.yf_ads.activity.YFSplashActivity;
import com.yifan.yf_ads.activity.YFSplashEventSink;
import com.yifan.yf_ads.admanager.InitAdSdkManager;
import com.yifan.yf_ads.admanager.InteractionAdManager;
import com.yifan.yf_ads.admanager.RewardVideoAdManager;
import com.yifan.yf_ads.bean.InitAdBean;
import com.yifan.yf_ads.bean.InteractionBean;
import com.yifan.yf_ads.bean.RewardVideoBean;
import com.yifan.yf_ads.bean.SplashBean;
import com.yifan.yf_ads.utils.Constants;
import com.yifan.yf_ads.utils.FLog;
import com.yifan.yf_ads.views.banner.BannerViewFactory;
import com.yifan.yf_ads.views.draw.DrawFeedVideoViewFactory;
import com.yifan.yf_ads.views.express.ExpressViewFactory;
import com.yifan.yf_ads.views.stream.StreamViewFactory;

import io.flutter.Log;
import io.flutter.embedding.android.FlutterActivity;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

/**
 * yf_adsPlugin
 */
public class YfAdsPlugin implements FlutterPlugin, MethodCallHandler, ActivityAware {

    private static final String TAG = "YfAdsPlugin";

    // The MethodChannel that will the communication between Flutter and native Android
    private MethodChannel channel;

    private FlutterActivity activity;

    private Context context;

    private InitAdSdkManager initAdSdkManager;

    private InteractionAdManager interactionAdManager;

    private RewardVideoAdManager rewardVideoAdManager;

    private BannerViewFactory bannerViewFactory;
    private DrawFeedVideoViewFactory drawFeedVideoViewFactory;

    private ExpressViewFactory expressFeedViewFactory;

    private StreamViewFactory streamViewFactory;

    private Application application;
    private EventChannel.EventSink eventSink;

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
        FLog.i(TAG, "onAttachedToEngine == ");
        channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "yf_ads_plugin");
        channel.setMethodCallHandler(this);
        initBannerViewFactory(flutterPluginBinding);
        initExpressFeedViewFactory(flutterPluginBinding);
        initDrawFeedVideoViewFactory(flutterPluginBinding);
        initStreamVideoViewFactory(flutterPluginBinding);
        final EventChannel eventChannel = new EventChannel(flutterPluginBinding.getBinaryMessenger(), "yf_ads_plugin_event");
        eventChannel.setStreamHandler(new EventChannel.StreamHandler() {
            @Override
            public void onListen(Object arguments, EventChannel.EventSink events) {
                FLog.i(TAG, "onListen");
                YfAdsPlugin.this.eventSink = events;
                if (bannerViewFactory != null) {
                    bannerViewFactory.setEventSink(events);
                }
                if (expressFeedViewFactory != null) {
                    expressFeedViewFactory.setEventSink(events);
                }
                if (drawFeedVideoViewFactory != null) {
                    drawFeedVideoViewFactory.setEventSink(events);
                }
                if (streamViewFactory != null) {
                    streamViewFactory.setEventSink(events);
                }
                events.success("Event data");
            }

            @Override
            public void onCancel(Object arguments) {
                YfAdsPlugin.this.eventSink = null;
            }
        });

        context = flutterPluginBinding.getApplicationContext();
        FLog.i(TAG, "context == " + context);
        if (context instanceof Application) {
            application = (Application) context;
            initAdSdkManager = InitAdSdkManager.getSingleton();
        }
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        String method = call.method;
        FLog.i(TAG + " method: " + method);
        if (!method.equals(Constants.KEY_METHOD_INIT_SDK)) {
            if (activity == null) {
                result.error(Constants.KEY_ERROR_INIT_FAIL, "activity do not attached", null);
                return;
            }
        }
        
        if (method.equals(Constants.KEY_METHOD_INIT_SDK)) {
            initSdk(call, result);
        } else if (method.equals(Constants.KEY_METHOD_SPLASH_AD)) {
            startSplash(call, result);
        } else if (method.equals(Constants.KEY_METHOD_REQUEST_INTERACTION_AD)) {
            startInteractionAd(call, result);
        } else if (method.equals(Constants.KEY_METHOD_REQUEST_REWARD_VIDEO_AD)) {
            startRewardVideoAd(call, result);
        } else if (method.equals(Constants.KEY_METHOD_BANNER_PAGE_CLOSED)) {
            stopBannerPageClosedAd();
        } else if (method.equals(Constants.KEY_METHOD_EXPRESS_FEED_AD_PAGE_CLOSED)) {
            stopExpressPageClosedAd();
        } else if (method.equals(Constants.KEY_METHOD_NATIVE_DRAW_FEED_VIDEO_AD_PAGE_CLOSED)) {
            stopDrawPageClosedAd();
        } else if (method.equals(Constants.KEY_METHOD_STREAM_AD_PAGE_CLOSED)) {
            stopStreamCloseAd();
        }
    }

    private void stopStreamCloseAd() {
        if (streamViewFactory != null) {
            streamViewFactory.releaseStreamViewCache();
        }
    }

    private void stopDrawPageClosedAd() {
        if (drawFeedVideoViewFactory != null) {
            drawFeedVideoViewFactory.releaseDrawFeedVideoViewCache();
        }
    }

    private void stopExpressPageClosedAd() {
        if (expressFeedViewFactory != null) {
            expressFeedViewFactory.releaseExpressFeedViewCache();
        }
    }

    private void stopBannerPageClosedAd() {
        if (bannerViewFactory != null) {
            bannerViewFactory.releaseBannerViewCache();
        }
    }

    private void startRewardVideoAd(MethodCall call, Result result) {
        try {
            String parameters = (String) call.arguments;
            RewardVideoBean rewardVideoBean = RewardVideoBean.parseObject(parameters);
            if (rewardVideoAdManager == null) {
                rewardVideoAdManager = new RewardVideoAdManager(activity);
            }
            rewardVideoAdManager.requestRewardVideoAd(rewardVideoBean, eventSink);
        } catch (Throwable e) {
            result.error(Constants.KEY_ERROR_REWARD_VIDEO_PARSE, "arguments parameters parse error", null);
            e.printStackTrace();
        }
    }

    private void startInteractionAd(MethodCall call, Result result) {
        try {
            String parameters = (String) call.arguments;
            InteractionBean interactionBean = InteractionBean.parseObject(parameters);
            if (interactionAdManager == null) {
                interactionAdManager = new InteractionAdManager(activity);
            }
            String positionId = interactionBean.getPositionId();
            interactionAdManager.requestInteractAD(positionId, eventSink);
        } catch (Throwable e) {
            result.error(Constants.KEY_ERROR_INTERACTION_PARSE, "arguments parameters parse error", null);
            e.printStackTrace();
        }
    }

    private void startSplash(@NonNull MethodCall call, @NonNull Result result) {
        try {
            String parameters = (String) call.arguments;
            SplashBean splashBean = SplashBean.parseObject(parameters);
            FLog.i("startSplash " + splashBean);
            String positionId = splashBean.getPositionId();
            String destActivityFullPath = splashBean.getDestActivityFullPath();
            String splashClassPath = splashBean.getSplashClassPath();
            Intent intent;
            if (splashClassPath != null && splashClassPath.length() > 0) {
                try {
                    Class<?> realClass = Class.forName(splashClassPath);
                    intent = new Intent(activity, realClass);
                } catch (Throwable e) {
                    intent = new Intent(activity, YFSplashActivity.class);
                }
            } else {
                intent = new Intent(activity, YFSplashActivity.class);
            }
            YFSplashEventSink.setEventSink(eventSink);
            intent.putExtra(Constants.KEY_POSITION_ID, positionId);
            intent.putExtra(Constants.KEY_DEST_ACTIVITY_FULL_PATH, destActivityFullPath);
            activity.startActivity(intent);
        } catch (Throwable e) {
            result.error(Constants.KEY_ERROR_SPLASH_PARSE, "arguments parameters parse error", null);
            e.printStackTrace();
        }
    }

    private void initSdk(@NonNull MethodCall call, @NonNull Result result) {
        String parameters = (String) call.arguments;
        if (parameters == null || parameters.length() == 0) {
            result.error(Constants.KEY_ERROR_INIT_FAIL, "parameters = null", null);
            return;
        }
        if (application == null) {
            result.error(Constants.KEY_ERROR_INIT_FAIL, "application is null", null);
            return;
        }
        try {
            FLog.i(TAG, "parameters = " + parameters);
            InitAdBean initAdBean = InitAdBean.parseObject(parameters);
            FLog.i(TAG, "initAdBean：" + initAdBean);
            initAdSdkManager.initSdk(application, initAdBean);
            result.success("success");
        } catch (Throwable e) {
            FLog.e(TAG, "initSdk：" + e.getMessage());
            e.printStackTrace();
            result.error(Constants.KEY_ERROR_INIT_FAIL, "init parameters parse error", null);
        }
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        channel.setMethodCallHandler(null);
    }

    @Override
    public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
        Log.i(TAG, "onAttachedToActivity");
        activity = (FlutterActivity) binding.getActivity();
        if (context == null) {
            context = activity.getApplication();
        }
        if (bannerViewFactory != null) {
            bannerViewFactory.setActivity(activity);
        }
        if (expressFeedViewFactory != null) {
            expressFeedViewFactory.setActivity(activity);
        }
        if (drawFeedVideoViewFactory != null) {
            drawFeedVideoViewFactory.setActivity(activity);
        }
        if (streamViewFactory != null) {
            streamViewFactory.setActivity(activity);
        }
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {

    }

    @Override
    public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {

    }

    @Override
    public void onDetachedFromActivity() {
        activity = null;
    }

    private void initBannerViewFactory(FlutterPluginBinding flutterPluginBinding) {
        FLog.i(TAG, "initBannerViewFactory");
        bannerViewFactory = new BannerViewFactory();
        flutterPluginBinding
                .getFlutterEngine()
                .getPlatformViewsController()
                .getRegistry()
                .registerViewFactory("com.yf/banner_platform_view", bannerViewFactory);
    }

    private void initExpressFeedViewFactory(FlutterPluginBinding flutterPluginBinding) {
        FLog.i(TAG, "initExpressFeedViewFactory");
        expressFeedViewFactory = new ExpressViewFactory();
        flutterPluginBinding
                .getFlutterEngine()
                .getPlatformViewsController()
                .getRegistry()
                .registerViewFactory("com.yf/express_feed_platform_view", expressFeedViewFactory);
    }


    private void initDrawFeedVideoViewFactory(FlutterPluginBinding flutterPluginBinding) {
        FLog.i(TAG, "initDrawFeedVideoViewFactory");
        drawFeedVideoViewFactory = new DrawFeedVideoViewFactory();
        flutterPluginBinding
                .getFlutterEngine()
                .getPlatformViewsController()
                .getRegistry()
                .registerViewFactory("com.yf/native_draw_feed_video_platform_view", drawFeedVideoViewFactory);
    }

    private void initStreamVideoViewFactory(FlutterPluginBinding flutterPluginBinding) {
        FLog.i(TAG, "initStreamVideoViewFactory");
        streamViewFactory = new StreamViewFactory();
        flutterPluginBinding
                .getFlutterEngine()
                .getPlatformViewsController()
                .getRegistry()
                .registerViewFactory("com.yf/stream_ad_platform_view", streamViewFactory);
    }

}
