package com.yifan.yf_ads.admanager;

import android.app.Activity;
import android.content.Context;
import android.view.ViewGroup;

import com.yfanads.android.core.banner.YFAdBanner;
import com.yfanads.android.core.banner.YFBannerListener;
import com.yfanads.android.model.YFAdError;
import com.yfanads.android.utils.ScreenUtil;
import com.yifan.yf_ads.bean.BannerAdBean;
import com.yifan.yf_ads.utils.Constants;
import com.yifan.yf_ads.utils.FLog;

import org.json.JSONException;
import org.json.JSONObject;

import io.flutter.plugin.common.EventChannel;

public class BannerAdManager extends CommonAdManager {
    private static final String TAG = "BannerAdManager";

    private YFAdBanner showADBanner;

    public BannerAdManager(Activity context) {
        super(context, Constants.AD_TYPE_BANNER);
    }

    public void requestBannerAd(final int viewId, final BannerAdBean bannerAdBean, final EventChannel.EventSink eventSink) {
        if (!isSDKInit(TAG)) {
            return;
        }
        String adPositionId = bannerAdBean.getPositionId();
        int width = bannerAdBean.getWidth();
        int height = bannerAdBean.getHeight();

        release();

        showADBanner = new YFAdBanner(activity, new YFBannerListener() {

            @Override
            public void onAdSuccess() {
                try {
                    FLog.i(TAG + " onAdSuccess");
                    callAdSuccess();
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put(Constants.KEY_METHOD, "onBannerPositionAdLoaded");
                    jsonObject.put(Constants.KEY_VIEW_ID, viewId);
                    callBack(jsonObject, eventSink);
                } catch (JSONException e) {
                    e.printStackTrace();
                }
            }

            @Override
            public void onAdExposure() {
                try {
                    FLog.i(TAG + " onAdExposure");
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put(Constants.KEY_METHOD, "onAdShow");
                    jsonObject.put(Constants.KEY_VIEW_ID, viewId);
                    callBack(jsonObject, eventSink);
                } catch (JSONException e) {
                    e.printStackTrace();
                }
            }

            @Override
            public void onAdClicked() {
                try {
                    FLog.i(TAG + " onAdClicked");
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put(Constants.KEY_METHOD, "onAdClicked");
                    jsonObject.put(Constants.KEY_VIEW_ID, viewId);
                    callBack(jsonObject, eventSink);
                } catch (JSONException e) {
                    e.printStackTrace();
                }
            }

            @Override
            public void onAdClosed() {
                try {
                    FLog.i(TAG + " onAdClosed");
                    callAdClose();
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put(Constants.KEY_METHOD, "onDislike");
                    jsonObject.put(Constants.KEY_VIEW_ID, viewId);
                    callBack(jsonObject, eventSink);
                } catch (JSONException e) {
                    e.printStackTrace();
                }
            }

            @Override
            public void onAdFailed(YFAdError yfAdError) {
                release();
                try {
                    FLog.i(TAG + " onAdFailed " + yfAdError.msg);
                    callAdFail(yfAdError.msg);
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put(Constants.KEY_METHOD, "onFailed");
                    jsonObject.put(Constants.KEY_ERROR, yfAdError.msg);
                    jsonObject.put(Constants.KEY_VIEW_ID, viewId);
                    callBack(jsonObject, eventSink);
                } catch (JSONException e) {
                    e.printStackTrace();
                }
            }
        });
        FLog.i(TAG, "requestBannerAd. width: " + width + ",height:" + height);
        Context context = activity.getApplicationContext();
        int acceptWidth = width > 0 ? width : ScreenUtil.px2dip(context, ScreenUtil.getScreenWidth(context));
        showADBanner.setViewAcceptedSize(acceptWidth, height);
        showADBanner.loadOnly(adPositionId);
    }

    public void showAd(Activity activity, ViewGroup viewGroup) {
        FLog.i(TAG, "showAd activity = " + activity + " ,viewGroup " + viewGroup);
        if (showADBanner != null && activity != null) {
            showADBanner.showAds(activity, viewGroup);
        }
    }

    public void release() {
        if (showADBanner != null) {
            showADBanner.destroy();
        }
    }

    public void destroy() {
        super.destroy();
        FLog.i(TAG, "destroy banner ad");
        release();
    }
}
