package com.yifan.yf_ads.admanager;

import android.app.Activity;
import android.graphics.Color;
import android.util.Log;
import android.view.View;
import android.view.ViewGroup;
import android.view.ViewTreeObserver;
import android.widget.LinearLayout;

import com.yfanads.android.core.nat.YFAdNativeExpressAds;
import com.yfanads.android.core.nat.YFNativeExpressListener;
import com.yfanads.android.model.ExpView;
import com.yfanads.android.model.NatExpView;
import com.yfanads.android.model.YFAdError;
import com.yfanads.android.utils.YFListUtils;
import com.yifan.yf_ads.R;
import com.yifan.yf_ads.bean.ExpressAdBean;
import com.yifan.yf_ads.utils.Constants;
import com.yifan.yf_ads.utils.DeviceUtils;
import com.yifan.yf_ads.utils.FLog;
import com.yifan.yf_ads.views.express.ExpressView;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.List;

import io.flutter.plugin.common.EventChannel;

/**
 * 一句话描述该类功能.
 *
 * @author JamesQian
 * @version 1.0
 * @copyright 亿帆
 * @date 2024/1/26 9:58
 **/
public class ExpressAdManager extends CommonAdManager {

    private static final String TAG = "NExpressAdManager";

    YFAdNativeExpressAds easyNativeExpress;

    public ExpressAdManager(Activity activity, String adType) {
        super(activity, adType);
    }

    public interface YFFlutterExpressFeedListener extends AdListener{
        void expressFeedAdLoadSuccess(ExpView view);
    }

    public void requestExpressFeedAd(final int viewId, final ExpressAdBean expressFeedAdBean,
                                     final EventChannel.EventSink eventSink) {
        if (!isSDKInit(TAG)) {
            return;
        }
        release();
        easyNativeExpress = new YFAdNativeExpressAds(activity, new YFNativeExpressListener() {

            @Override
            public void onAdRenderSuccess(List<ExpView> viewList) {

                if (YFListUtils.isEmpty(viewList)) {
                    FLog.i("view list is empty, return.");
                    return;
                }

                ExpView expView = viewList.get(0);
                if (expView != null && adListener instanceof YFFlutterExpressFeedListener) {
                    ((YFFlutterExpressFeedListener) adListener).expressFeedAdLoadSuccess(expView);
                }
                try {
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put(Constants.KEY_METHOD, "onNativeExpressAdLoaded");
                    jsonObject.put(Constants.KEY_VIEW_ID, viewId);
                    callBack(jsonObject, eventSink);
                } catch (JSONException e) {
                    e.printStackTrace();
                }
            }

            @Override
            public void onAdSuccess() {
                callAdSuccess();
            }

            @Override
            public void onAdRenderFailed() {
                try {
                    callAdFail("onRenderFail");
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put(Constants.KEY_METHOD, "onRenderFail");
                    jsonObject.put(Constants.KEY_ERROR, "onRenderFail");
                    jsonObject.put(Constants.KEY_VIEW_ID, viewId);
                    callBack(jsonObject, eventSink);
                } catch (JSONException e) {
                    e.printStackTrace();
                }
            }

            @Override
            public void onAdFailed(YFAdError fcAdError) {
                try {
                    callAdFail(fcAdError.msg);
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put(Constants.KEY_METHOD, "onFailed");
                    jsonObject.put(Constants.KEY_ERROR, fcAdError.msg);
                    jsonObject.put(Constants.KEY_VIEW_ID, viewId);
                    callBack(jsonObject, eventSink);
                } catch (JSONException e) {
                    e.printStackTrace();
                }
            }

            @Override
            public void onAdExposure(ExpView expView) {
                try {
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put(Constants.KEY_METHOD, "onAdShow");
                    jsonObject.put(Constants.KEY_VIEW_ID, viewId);
                    callBack(jsonObject, eventSink);
                } catch (JSONException e) {
                    e.printStackTrace();
                }
            }

            @Override
            public void onAdClicked(ExpView expView) {
                try {
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put(Constants.KEY_METHOD, "onAdClicked");
                    jsonObject.put(Constants.KEY_VIEW_ID, viewId);
                    callBack(jsonObject, eventSink);
                } catch (JSONException e) {
                    e.printStackTrace();
                }
            }

            @Override
            public void onAdClosed(ExpView expView) {
                try {
                    callAdClose();
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put(Constants.KEY_METHOD, "onDislike");
                    jsonObject.put(Constants.KEY_VIEW_ID, viewId);
                    callBack(jsonObject, eventSink);
                } catch (JSONException e) {
                    e.printStackTrace();
                }
            }
        });

        easyNativeExpress.setAdsNumbers(1);
        easyNativeExpress.loadOnly(expressFeedAdBean.getPositionId());
    }

    public void showAd(Activity activity) {
        FLog.i(TAG, "showAd activity = " + activity);
        if (easyNativeExpress != null && activity != null) {
            easyNativeExpress.showAds(activity);
        }
    }

    public void release() {
        if (easyNativeExpress != null) {
            easyNativeExpress.destroy();
        }
    }

    public void destroy() {
        super.destroy();
        FLog.i(TAG, "destroy express ad ");
        release();
    }
}
