package com.yifan.yf_ads.admanager;

import android.app.Activity;

import com.yfanads.android.core.reward.YFAdRewardAds;
import com.yfanads.android.core.reward.YFRewardServerCallBackInf;
import com.yfanads.android.core.reward.YFRewardVideoListener;
import com.yfanads.android.model.YFAdError;
import com.yfanads.android.utils.YFLog;
import com.yifan.yf_ads.bean.RewardVideoBean;
import com.yifan.yf_ads.utils.Constants;
import com.yifan.yf_ads.utils.FLog;

import org.json.JSONException;
import org.json.JSONObject;

import io.flutter.plugin.common.EventChannel;

/**
 * 一句话描述该类功能.
 *
 * @author JamesQian
 * @version 1.0
 * @copyright 亿帆
 * @date 2024/1/25 11:38
 **/
public class RewardVideoAdManager extends CommonAdManager {

    private static final String TAG = "RewardVideoAdManager";

    private YFAdRewardAds easyRewardVideo;

    public RewardVideoAdManager(Activity activity) {
        super(activity, Constants.AD_TYPE_REWARD_VIDEO);
    }

    public void requestRewardVideoAd(RewardVideoBean rewardVideoBean, EventChannel.EventSink eventSink) {
        if (!isSDKInit(TAG)) {
            return;
        }

        easyRewardVideo = new YFAdRewardAds(activity, new YFRewardVideoListener() {
            @Override
            public void onVideoCached() {
                try {
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put(Constants.KEY_METHOD, "onRewardVideoCached");
                    callBack(jsonObject, eventSink);
                } catch (JSONException e) {
                    e.printStackTrace();
                }
            }

            @Override
            public void onVideoComplete() {
                try {
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put(Constants.KEY_METHOD, "onVideoComplete");
                    callBack(jsonObject, eventSink);
                } catch (JSONException e) {
                    e.printStackTrace();
                }
            }

            @Override
            public void onVideoSkip() {
                try {
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put(Constants.KEY_METHOD, "onSkippedVideo");
                    callBack(jsonObject, eventSink);
                } catch (JSONException e) {
                    e.printStackTrace();
                }
            }

            @Override
            public void onRewardServerInf(YFRewardServerCallBackInf yfRewardServerCallBackInf) {

                try {
                    FLog.i(TAG, "onRewardVerify. rewardVerify: " + yfRewardServerCallBackInf.rewardInf.type);
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put(Constants.KEY_METHOD, "onRewardVerify");
                    jsonObject.put(Constants.KEY_REWARD_NAME, getRewardName(yfRewardServerCallBackInf.rewardInf.type));
                    jsonObject.put(Constants.KEY_REWARD_VERIFY, true);
                    callBack(jsonObject, eventSink);
                } catch (JSONException e) {
                    e.printStackTrace();
                }
            }

            @Override
            public void onAdSuccess() {
                try {
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put(Constants.KEY_METHOD, "onAdSuccess");
                    callBack(jsonObject, eventSink);
                } catch (JSONException e) {
                    e.printStackTrace();
                }
                showAds(activity);
            }

            @Override
            public void onAdExposure() {
                try {
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put(Constants.KEY_METHOD, "onAdShow");
                    callBack(jsonObject, eventSink);
                } catch (JSONException e) {
                    e.printStackTrace();
                }
            }

            @Override
            public void onAdClicked() {
                try {
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put(Constants.KEY_METHOD, "onAdClicked");
                    callBack(jsonObject, eventSink);
                } catch (JSONException e) {
                    e.printStackTrace();
                }
            }

            @Override
            public void onAdClosed() {
                // 广告关闭
                try {
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put(Constants.KEY_METHOD, "onAdClose");
                    callBack(jsonObject, eventSink);
                } catch (JSONException e) {
                    e.printStackTrace();
                }
            }

            @Override
            public void onAdFailed(YFAdError yfAdError) {
                try {
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put(Constants.KEY_METHOD, "onFailed");
                    jsonObject.put(Constants.KEY_ERROR, yfAdError.msg);
                    callBack(jsonObject, eventSink);
                } catch (JSONException e) {
                    e.printStackTrace();
                }
            }
        }, rewardVideoBean.getUserId());
        easyRewardVideo.setAppExtra("extra", rewardVideoBean.getExtra());
        easyRewardVideo.setOrientation(rewardVideoBean.getOrientation());
        easyRewardVideo.loadOnly(rewardVideoBean.getPositionId());
    }

    private void showAds(Activity activity) {
        if (easyRewardVideo != null) {
            easyRewardVideo.showAds(activity);
        }
    }

    private String getRewardName(YFRewardServerCallBackInf.Type type) {
        switch (type) {
            case ADX:
                return "yifan";
            case BD:
                return "baidu";
            case CSJ:
                return "csj";
            case YLH:
                return "ylh";
            case KS:
                return "ks";
            case HW:
                return "hw";
            case OPPO:
                return "oppo";
            case VIVO:
                return "vivo";
            case XM:
                return "xm";
        }
        return "";
    }

    @Override
    protected void destroy() {
        super.destroy();
        if (easyRewardVideo != null) {
            easyRewardVideo.destroy();
        }
    }
}
