package com.yifan.yf_ads.admanager;

import android.app.Activity;
import android.content.Context;
import android.view.View;
import android.view.ViewGroup;

import com.yfanads.android.core.roll.YFRollAds;
import com.yfanads.android.core.roll.YFRollAdsListener;
import com.yfanads.android.model.YFAdError;
import com.yfanads.android.utils.ScreenUtil;
import com.yfanads.android.utils.YFListUtils;
import com.yifan.yf_ads.bean.StreamAdBean;
import com.yifan.yf_ads.utils.Constants;
import com.yifan.yf_ads.utils.FLog;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.List;

import io.flutter.plugin.common.EventChannel;

public class StreamAdManager extends CommonAdManager {
    private static final String TAG = "StreamAdManager";

    private YFRollAds rollAds;

    private ViewGroup adView;

    public StreamAdManager(Activity activity, String adType) {
        super(activity, adType);
    }


    public void requestStreamAd(final int viewId, final StreamAdBean streamAdBean, final EventChannel.EventSink eventSink) {
        if (!isSDKInit(TAG)) {
            return;
        }
        release();

        String adPositionId = streamAdBean.getPositionId();
        int width = streamAdBean.getWidth();
        int height = streamAdBean.getHeight();
        rollAds = new YFRollAds(activity, new YFRollAdsListener() {
            @Override
            public void onAdSuccess() {
                callAdSuccess();
                try {
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put(Constants.KEY_METHOD, "onAdLoadedNative");
                    jsonObject.put(Constants.KEY_VIEW_ID, viewId);
                    callBack(jsonObject, eventSink);
                } catch (JSONException e) {
                    e.printStackTrace();
                }
            }

            @Override
            public void onAdExposure() {
                try {
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put(Constants.KEY_METHOD, "onAdShow");
                    jsonObject.put(Constants.KEY_VIEW_ID, viewId);
                    callBack(jsonObject, eventSink);
                } catch (JSONException e) {
                    e.printStackTrace();
                }
            }

            @Override
            public void onAdClicked() {
                try {
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put(Constants.KEY_METHOD, "onAdClicked");
                    jsonObject.put(Constants.KEY_VIEW_ID, viewId);
                    callBack(jsonObject, eventSink);
                } catch (JSONException e) {
                    e.printStackTrace();
                }
            }

            @Override
            public void onAdClosed() {
                callAdClose();
            }

            @Override
            public void onAdFailed(YFAdError yfAdError) {
                try {
                    callAdFail(yfAdError.msg);
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put(Constants.KEY_METHOD, "onFailed");
                    jsonObject.put(Constants.KEY_ERROR, yfAdError.msg);
                    jsonObject.put(Constants.KEY_VIEW_ID, viewId);
                    callBack(jsonObject, eventSink);
                } catch (JSONException e) {
                    e.printStackTrace();
                }
            }

            @Override
            public void onAdRenderFailed() {
                try {
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put(Constants.KEY_METHOD, "onRenderFail");
                    jsonObject.put(Constants.KEY_ERROR, "onRenderFail");
                    jsonObject.put(Constants.KEY_VIEW_ID, viewId);
                    callBack(jsonObject, eventSink);
                    destroy();
                } catch (JSONException e) {
                    e.printStackTrace();
                }
            }

            @Override
            public void onAdRenderSuccess(List<View> addViewList) {

                if (!YFListUtils.isEmpty(addViewList) && adView != null) {
                    adView.removeAllViews();
                    adView.addView(addViewList.get(0));
                }

                try {
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put(Constants.KEY_METHOD, "onRenderSuccess");
                    jsonObject.put(Constants.KEY_VIEW_ID, viewId);
                    callBack(jsonObject, eventSink);
                } catch (JSONException e) {
                    e.printStackTrace();
                }
            }
        });
        FLog.i(TAG, "requestStreamAd. width: " + width + ",height:" + height);
        Context context = activity.getApplicationContext();
        int acceptWidth = width > 0 ? width : ScreenUtil.px2dip(context, ScreenUtil.getScreenWidth(context));
        rollAds.setViewAcceptedSize(acceptWidth, height);
        rollAds.loadOnly(adPositionId);
    }

    public void showAds(Activity activity, ViewGroup adView) {
        if (rollAds != null && activity != null) {
            this.adView = adView;
            rollAds.showAds(activity, adView);
        }
    }

    private void release() {
        if (rollAds != null) {
            rollAds.destroy();
        }
    }

    public void destroy() {
        super.destroy();
        release();
    }
}
