package com.yifan.yf_ads.views.banner;

import android.app.Activity;
import android.content.Context;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.yifan.yf_ads.bean.BannerAdBean;
import com.yifan.yf_ads.utils.FLog;

import java.util.HashMap;
import java.util.Map;

import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.MessageCodec;
import io.flutter.plugin.common.StandardMessageCodec;
import io.flutter.plugin.platform.PlatformView;
import io.flutter.plugin.platform.PlatformViewFactory;
/**
 * 一句话描述该类功能.
 *
 * @author JamesQian
 * @version 1.0
 * @copyright 亿帆
 * @date 2024/1/24 9:33
 **/
public class BannerViewFactory extends PlatformViewFactory {
    private static final String TAG = "BannerViewFactory";
    private Map<Integer, BannerView> bannerViewMap;

    private Activity activity;

    private EventChannel.EventSink eventSink;

    public BannerViewFactory() {
        super(StandardMessageCodec.INSTANCE);
        FLog.i(TAG, "BannerViewFactory construct");
        bannerViewMap = new HashMap<>();
    }
    /**
     * @param createArgsCodec the codec used to decode the args parameter of {@link #create}.
     */
    public BannerViewFactory(@Nullable MessageCodec<Object> createArgsCodec) {
        super(createArgsCodec);
    }

    public void setActivity(Activity activity) {
        FLog.i(TAG, "setActivity");
        this.activity = activity;
    }

    public void setEventSink(EventChannel.EventSink eventSink) {
        FLog.i(TAG, "setEventSink");
        this.eventSink = eventSink;
    }

    public void releaseBannerViewCache() {
        FLog.i(TAG, "releaseBannerViewCache");
        if (bannerViewMap != null) {
            for (Map.Entry<Integer, BannerView> entry : bannerViewMap.entrySet()) {
                BannerView childView = entry.getValue();
                if (childView != null) {
                    childView.releaseAd();
                }
            }
            bannerViewMap.clear();
        }
    }

    @NonNull
    @Override
    public PlatformView create(@Nullable Context context, int viewId, @Nullable Object args) {
        if (activity == null) {
            FLog.e(TAG, "activity is null");
            return null;
        }
        try {
            final String parameters = (String) args;
            BannerAdBean creationParams = BannerAdBean.parseObject(parameters);
            int uniqueKey = creationParams.getUniqueKey();
            FLog.i(TAG, "create uniqueKey:" + uniqueKey);
            if (bannerViewMap.containsKey(uniqueKey)) {
                BannerView cachedView = bannerViewMap.get(uniqueKey);
                if (cachedView != null) {
                    FLog.i(TAG, "hit cached banner view");
                    return cachedView;
                }
            }
            BannerView bannerView = new BannerView(activity, uniqueKey, creationParams, this.eventSink);
            bannerViewMap.put(uniqueKey, bannerView);
            return bannerView;
        } catch (Throwable e) {
            e.printStackTrace();
            FLog.e(TAG, "bannerView Factory " + e.getMessage());
        }
        return null;
    }
}
