package com.yifan.yf_ads.views.express;

import android.app.Activity;
import android.graphics.Color;
import android.view.View;
import android.view.ViewGroup;
import android.view.ViewTreeObserver;
import android.widget.LinearLayout;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.yfanads.android.model.ExpView;
import com.yifan.yf_ads.admanager.CommonAdManager;
import com.yifan.yf_ads.admanager.ExpressAdManager;
import com.yifan.yf_ads.bean.ExpressAdBean;
import com.yifan.yf_ads.utils.Constants;
import com.yifan.yf_ads.utils.DeviceUtils;
import com.yifan.yf_ads.utils.FLog;

import org.json.JSONObject;

import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.platform.PlatformView;

/**
 * 一句话描述该类功能.
 *
 * @author JamesQian
 * @version 1.0
 * @copyright 亿帆
 * @date 2024/1/26 10:30
 **/
public class ExpressView implements PlatformView {

    private final static String TAG = "ExpressFeedView";

    private ExpressAdManager expressAdManager;

    @NonNull
    private LinearLayout containerView;

    ExpressView(@NonNull final Activity context, final int id, @Nullable
    final ExpressAdBean creationParams, final EventChannel.EventSink eventSink) {
        FLog.i(TAG, "NativeView id = " + id);
        if (creationParams == null) {
            return;
        }
        if (expressAdManager == null) {
            expressAdManager = new ExpressAdManager(context, Constants.AD_TYPE_EXPRESS_FEED);
        }

        containerView = new LinearLayout(context);
        expressAdManager.setAdListener(new ExpressAdManager.YFFlutterExpressFeedListener() {
            @Override
            public void expressFeedAdLoadSuccess(ExpView view) {
                FLog.i(TAG, "expressFeedAdLoadSuccess ");
                if (view == null) {
                    return;
                }
                if (view.getView() != null) {
                    onContainerSizeChanged(view.getView(), id, eventSink);
                } else {
                    view.render(containerView);
                }
            }

            @Override
            public void onAdClose() {
                FLog.i(TAG, "onAdClose ");
                containerView.removeAllViews();
                ViewGroup.LayoutParams layoutParams = containerView.getLayoutParams();
                layoutParams.height = 0;
                containerView.setLayoutParams(layoutParams);
            }

            @Override
            public void onAdLoadSuccess() {
                expressAdManager.showAd(context);
            }

            @Override
            public void onAdLoadFail(String errorMessage) {
                FLog.i(TAG, "expressFeedAdLoadFail:" + errorMessage);
            }
        });
        expressAdManager.requestExpressFeedAd(id, creationParams, eventSink);
    }

    private void onContainerSizeChanged(View view, int viewId, final EventChannel.EventSink eventSink) {
        view.getViewTreeObserver().addOnGlobalLayoutListener(new ViewTreeObserver.OnGlobalLayoutListener() {

            @Override
            public void onGlobalLayout() {
                int width = view.getWidth();
                int height = view.getHeight();
                FLog.i(TAG, "expressFeedAdLoadSuccess width:" + width + ",height:" + height + " , " + viewId);
                if (height > 1) {
                    try {
                        int tempHeight = DeviceUtils.px2dp(view.getContext(), height);
                        FLog.i(TAG, "expressFeedAdLoadSuccess tempHeight:" + tempHeight);
                        JSONObject jsonObject = new JSONObject();
                        jsonObject.put(Constants.KEY_METHOD, "onContainerSizeChanged");
                        jsonObject.put(Constants.KEY_VIEW_ID, viewId);
                        jsonObject.put(Constants.KEY_EXPRESS_FEED_VIEW_HEIGHT, tempHeight);
                        jsonObject.put(Constants.KEY_AD_TYPE, Constants.AD_TYPE_EXPRESS_FEED);
                        eventSink.success(jsonObject.toString());
                    } catch (Throwable e) {
                        FLog.i(TAG, "expressFeedAdLoadSuccess error:" + e.getMessage());
                    }
                    ViewGroup.LayoutParams layoutParams = containerView.getLayoutParams();
                    layoutParams.height = height;
                    containerView.setLayoutParams(layoutParams);
                    view.getViewTreeObserver().removeOnGlobalLayoutListener(this);
                    FLog.i(TAG, "expressFeedAdLoadSuccess height:" + height);
                }
            }
        });
        containerView.removeAllViews();
        containerView.addView(view);
    }

    @Override
    public View getView() {
        return containerView;
    }

    @Override
    public void dispose() {
        FLog.i(TAG, "dispose");
//        releaseAd();
    }

    public void releaseAd() {
        if (expressAdManager != null) {
            expressAdManager.destroy();
        }
    }
}
