package com.yifan.yf_ads.views.stream;

import android.app.Activity;
import android.content.Context;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;


import com.yifan.yf_ads.bean.StreamAdBean;
import com.yifan.yf_ads.utils.FLog;

import java.util.HashMap;
import java.util.Map;

import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.StandardMessageCodec;
import io.flutter.plugin.platform.PlatformView;
import io.flutter.plugin.platform.PlatformViewFactory;


public class StreamViewFactory extends PlatformViewFactory {
    private EventChannel.EventSink eventSink;
    private Activity activity;
    private final String TAG = "StreamViewFactory";
    private final Map<Integer, StreamView> streamViewMap;

    public StreamViewFactory() {
        super(StandardMessageCodec.INSTANCE);
        FLog.i(TAG, "StreamViewFactory construct");
        streamViewMap = new HashMap<>();
    }

    public void setActivity(Activity activity) {
        FLog.i(TAG, "setActivity");
        this.activity = activity;
    }

    public void setEventSink(EventChannel.EventSink eventSink) {
        FLog.i(TAG, "setEventSink");
        this.eventSink = eventSink;
    }

    public void releaseStreamViewCache() {
        FLog.i(TAG, "releaseStreamViewCache");
        if (streamViewMap != null) {
            for (Map.Entry<Integer, StreamView> entry : streamViewMap.entrySet()) {
                StreamView childView = entry.getValue();
                if (childView != null) {
                    childView.releaseAd();
                }
            }
            streamViewMap.clear();
        }
    }

    @NonNull
    @Override
    public PlatformView create(@NonNull Context context, int id, @Nullable Object args) {
        if (activity == null) {
            FLog.i(TAG, "activity == null");
            return null;
        }
        try {
            final String parameters = (String) args;
            final StreamAdBean creationParams = StreamAdBean.parseObject(parameters);
            int uniqueKey = creationParams.getUniqueKey();
            FLog.i(TAG, "create uniqueKey:" + uniqueKey);
            if (streamViewMap.containsKey(uniqueKey)) {
                StreamView cachedView = streamViewMap.get(uniqueKey);
                if (cachedView != null) {
                    FLog.i(TAG, "hit cached native view");
                    return cachedView;
                }
            }
            StreamView nativeView = new StreamView(activity, uniqueKey, creationParams, this.eventSink);
            streamViewMap.put(uniqueKey, nativeView);
            return nativeView;
        } catch (Exception exception) {
            exception.printStackTrace();
        }
        return null;
    }
}
