# 亿帆Flutter插件接入文档


# 1. Flutter插件环境
## 1.1 接入须知
如果您准备接入亿帆flutter插件，我们假设负责接入的开发人员具备了一定的Android和iOS开发能力。

## 1.2 接入环境配置
亿帆插件基于最新发布的flutter及dart版本开发，一些新的特性被用到了当前插件中，例如空安全。

当前开发机器的配置如下：
````
Flutter 3.0.0 • channel stable • https://github.com/flutter/flutter.git
Framework • revision ee4e09cce0 (1 year, 11 months ago) • 2022-05-09 16:45:18 -0700
Engine • revision d1b9a6938a
Tools • Dart 2.17.0 • DevTools 2.12.2
````

## 1.3 安装Flutter环境
安装环境见如下网址：

https://flutter.cn/docs/get-started/install，请按照原生代码所在平台，安装对应平台的flutter sdk。

在Android Studio中，请分别安装dart和Flutter插件。

## 1.4 接入万汇Flutter插件
当前插件版本发布在https://pub.dev/packages/yf_ads，请务必接入当前网站中最新的插件版本。

在您的flutter工程根文件夹下，找到pubspec.yaml文件，添加如下依赖：

````
dependencies:
  yf_ads: 5.3.3
````
然后到工程根目录下执行：

````
flutter pub get
````
执行成功之后就可以开始接入。

# 2 Android基础配置
## 2.1 前置条件
如果您准备在Android设备展示广告，请将flutter工程重新以Android工程形式打开。

如果您之前接入了其中任何一个广告或其依赖的sdk，请务必替换成亿帆的版本，否则广告可能在填充或展示的过程中出现不可控的错误。

## 2.2 申请应用广告位
请联系亿帆商务人员，申请应用ID和广告位ID。

## 2.3 导入aar及SDK依赖的jar包
### 2.3.1 添加仓库
在项目的build.gradle中添加如下代码：
````
repositories {
    maven { url "https://maven.yfanads.com/repository/maven-public/"}
}
````
将本SDK压缩包内依赖的三方SDK文件夹中的aar文件复制到Application Module/libs文件夹(没有的话须手动创建), 并将以下代码添加到您app的build.gradle中：

````
repositories {
    flatDir {
        dirs 'libs'
    }
}
````

### 2.3.2 添加依赖
#### AndroidX版本</br>
因为亿帆SDK还没有使用AndroidX库，为了避免应用程序崩溃，请在项目的gradle.properties中添加android.enableJetifier=true 和android.useAndroidX=true 来打开Jetifier。

如下是亿帆flutter版本需要依赖的sdk，请将对应的本地sdk版本修改成提供的最新版本即可。

````
    implementation("com.yfanads.ads:sdk:5.3.3.8")
    implementation("com.yfanads.ads:sdk-core:5.3.3.8")
    // 如果接入厂家SDK（华为、小米、VIVO、OPPO）需要导入
    implementation("com.yfanads.ads:sdk-factory:5.3.3.8")
    // 百度SDK
    implementation(name: "Baidu_MobAds_SDK-release_v9.34", ext: "aar")
    
    // 穿上甲SDK
    implementation 'com.pangle.cn:ads-sdk-pro:5.9.1.4'
    implementation 'com.squareup.okhttp3:okhttp:3.12.1'
    
    // 快手SDK
    implementation(name: "kssdk-ad-33612", ext: "aar")
    implementation 'androidx.localbroadcastmanager:localbroadcastmanager:1.1.0'
    
    // 优量汇SDK
    implementation 'com.qq.e.union:union:4.563.1433'
    
    // 京东SDK
    implementation(name: "jad_yun_sdk_jingdong_2.5.10_20240130", ext: "aar")
    
    // 华为 (build需要新增maven库)
    implementation "com.huawei.hms:ads-lite:13.4.69.300"
    
    // oppo
    implementation(name: "mobad_normal_pub_563004_2023_10_18", ext: "aar")
    implementation 'androidx.appcompat:appcompat:1.0.2'
    implementation 'androidx.legacy:legacy-support-v4:1.0.0'
    implementation 'androidx.multidex:multidex:2.0.0'
    implementation 'com.squareup.okio:okio:2.5.0'
    implementation 'org.jetbrains.kotlin:kotlin-android-extensions-runtime:1.3.72'
    implementation 'android.arch.persistence:db-framework:1.1.1'//410版本新增
    implementation 'androidx.palette:palette:1.0.0'//490版本新增
    //由于可能与接入业务产生冲突，故由开发者决定依赖的版本，wechat sdk区分wechat-sdk-android与wechat-sdk-android-without-mta,由开发者决定依赖版本
    implementation "com.tencent.mm.opensdk:wechat-sdk-android-without-mta:5.5.8"//注意480版本新增，可以提升广告效果
    implementation 'androidx.recyclerview:recyclerview:1.0.0'//注意500版本之后强要求依赖
    
    
    // 小米
    implementation(name: "mimo-ad-sdk-530", ext: "aar")
    implementation 'androidx.appcompat:appcompat:1.4.1'
    implementation 'androidx.recyclerview:recyclerview:1.0.0'
    implementation 'com.google.code.gson:gson:2.8.5'
    implementation 'com.github.bumptech.glide:glide:4.9.0'
    annotationProcessor 'com.github.bumptech.glide:compiler:4.9.0'
    
    // vivo
    implementation(name: "open_ad_6.1.5.1", ext: "aar")
````

## 2.4 AndroidManifest.xml配置
### 2.4.1 添加权限
````
    <!--必须要有的权限-->
    <uses-permission android:name="android.permission.INTERNET" />

    <!--强烈推荐权限-->
    <!--强烈建议您在AndroidManifest.xml添加以下权限声明，若您的targetSDKVersion >= 23您还需要在运行时进行动态权限申请。
    注意：SDK不强制校验以下权限（即:无以下权限sdk也可正常工作），但建议您申请以下权限。
          针对单媒体的用户，允许获取权限的，投放定向广告；
          不允许获取权限的用户，投放通投广告。
          媒体可以选择是否把以下权限提供出来，并承担相应广告填充和eCPM单价下降损失的结果。-->
    <uses-permission android:name="android.permission.READ_PHONE_STATE" />
    <uses-permission android:name="android.permission.ACCESS_COARSE_LOCATION" />
    <!-- 可选，如果需要精确定位的话请加上此权限 -->
    <uses-permission android:name="android.permission.ACCESS_FINE_LOCATION" />

    <!--可选权限-->
    <!--下面权限可以优化用户的交互体验，提高eCPM。-->
    <uses-permission android:name="android.permission.WRITE_EXTERNAL_STORAGE" />
    <uses-permission android:name="android.permission.ACCESS_NETWORK_STATE" />
    <uses-permission android:name="android.permission.ACCESS_WIFI_STATE" />
    <uses-permission android:name="android.permission.REQUEST_INSTALL_PACKAGES" />
    <!--注意: 该权限不强制获取，即使没有获取也能正常运行；获取该权限将帮助优化投放广告精准度。-->
    <uses-permission android:name="android.permission.SYSTEM_ALERT_WINDOW" />

    <!--穿山甲的必要权限，解决安全风险漏洞，发送和注册广播事件需要调用带有传递权限的接口-->
    <permission
        android:name="${applicationId}.openadsdk.permission.TT_PANGOLIN"
        android:protectionLevel="signature" />

    <uses-permission android:name="${applicationId}.openadsdk.permission.TT_PANGOLIN" />

    <!-- 如果有视频相关的广告且使用textureView播放，请务必添加，否则黑屏 -->
    <uses-permission android:name="android.permission.WAKE_LOCK" />
````

### 2.4.2 适配Android7.0及以上
#### 2.4.2.1 Provider组件
亿帆SDK使用FileProvider解决7.0以上系统APK安装失败问题。
````
    <!--亿帆-->
    <provider
        android:name="com.yfanads.android.adx.download.AdxFileProvider"
        android:authorities="${applicationId}.AdxFileProvider"
        android:exported="false"
        android:grantUriPermissions="true">
        <meta-data
            android:name="android.support.FILE_PROVIDER_PATHS"
            android:resource="@xml/adxfilepaths" />
    </provider>
````

### 2.4.3 添加组件
如果您接入了优量汇或者穿山甲或者小米，需要在AndroidManifest.xml中添加如下组件：
````
    <!--穿山甲-->
    <provider
        android:name="com.bytedance.sdk.openadsdk.TTFileProvider"
        android:authorities="${applicationId}.TTFileProvider"
        android:exported="false"
        android:grantUriPermissions="true">
        <meta-data
            android:name="android.support.FILE_PROVIDER_PATHS"
            android:resource="@xml/ea_csj_file_paths" />
    </provider>

    <provider
        android:name="com.bytedance.sdk.openadsdk.multipro.TTMultiProvider"
        android:authorities="${applicationId}.TTMultiProvider"
        android:exported="false" />

    <!--优量汇-->
    <provider
        android:name="com.qq.e.comm.GDTFileProvider"
        android:authorities="${applicationId}.gdt.fileprovider"
        android:exported="false"
        android:grantUriPermissions="true">
        <meta-data
            android:name="android.support.FILE_PROVIDER_PATHS"
            android:resource="@xml/gdt_file_path" />
    </provider>

    <!-- 小米 -->
    <provider
        android:name="androidx.core.content.FileProvider"
        android:authorities="${applicationId}.fileprovider"
        android:exported="false"
        android:grantUriPermissions="true">
        <meta-data
            android:name="android.support.FILE_PROVIDER_PATHS"
            android:resource="@xml/xm_file_paths" />
    </provider>
````

## 2.5 混淆问题

请添加下面混淆规则到应用混淆规则中。
````
-keepattributes Signature

# For using GSON @Expose annotation
-keepattributes *Annotation*

# Gson specific classes
-dontwarn sun.misc.**
#-keep class com.google.gson.stream.** { *; }

# Application classes that will be serialized/deserialized over Gson
-keep class com.google.gson.examples.android.model.** { <fields>; }

# Prevent proguard from stripping interface information from TypeAdapter, TypeAdapterFactory,
# JsonSerializer, JsonDeserializer instances (so they can be used in @JsonAdapter)
-keep class * extends com.google.gson.TypeAdapter
-keep class * implements com.google.gson.TypeAdapterFactory
-keep class * implements com.google.gson.JsonSerializer
-keep class * implements com.google.gson.JsonDeserializer

# Prevent R8 from leaving Data object members always null
-keepclassmembers,allowobfuscation class * {
  @com.google.gson.annotations.SerializedName <fields>;
}

# Retain generic signatures of TypeToken and its subclasses with R8 version 3.0 and higher.
-keep,allowobfuscation,allowshrinking class com.google.gson.reflect.TypeToken
-keep,allowobfuscation,allowshrinking class * extends com.google.gson.reflect.TypeToken

##---------------End: proguard configuration for Gson  ----------

## ------聚合开始
-keep class com.yfanads.android.** { *; }
-keep class com.yfanads.ads.** { *; }
## ------聚合结束

# -------三方依赖开始
-keep class com.google.** { *; }
-dontwarn okio.**
-dontwarn com.ak.**
-dontwarn android.webkit.**
-dontwarn com.bytedance.sdk.**
-keep class android.** { *; }
-keep public class * implements com.bumptech.glide.module.GlideModule
-keep public enum com.bumptech.glide.load.resource.bitmap.ImageHeaderParser$** {
  **[] $VALUES;
  public *;
}
# -------三方依赖结束

# csj sdk start
#3500以及以下版本混淆规则如下：
-keep class com.bytedance.sdk.openadsdk.** { *; }
-keep public interface com.bytedance.sdk.openadsdk.downloadnew.** {*;}
-keep class com.pgl.sys.ces.* {*;}
-keep class com.bytedance.embed_dr.** {*;}
-keep class com.bytedance.embedapplog.** {*;}
#3600版本-3800版本混淆规则如下：
-keep class com.bytedance.sdk.openadsdk.** { *; }
-keep public interface com.bytedance.sdk.openadsdk.downloadnew.** {*;}
-keep class ms.bd.c.**{*;}
-keep class com.bytedance.mobsec.**{*;}
-keep class com.bytedance.embed_dr.** {*;}
-keep class com.bytedance.embedapplog.** {*;}
-keep interface com.bytedance.frameworks.baselib.network.http.cronet.I* {*;}
-keepnames class com.bytedance.framwork.core.sdkmonitor.SDKMonitorUtils
#3900版本及以上版本混淆规则如下：
-keep class com.bytedance.sdk.openadsdk.** { *; }
-keep class com.bytedance.frameworks.** { *; }
-keep class ms.bd.c.Pgl.**{*;}
-keep class com.bytedance.mobsec.metasec.ml.**{*;}
-keep class com.ss.android.**{*;}
-keep class com.bytedance.embedapplog.** {*;}
-keep class com.bytedance.embed_dr.** {*;}
-keep class com.bykv.vk.** {*;}

-keep class bykvm*.**
-keep class com.bytedance.msdk.adapter.**{ public *; }
-keep class com.bytedance.msdk.api.** {
 public *;
}
-keep class com.bytedance.msdk.base.TTBaseAd{*;}
-keep class com.bytedance.msdk.adapter.TTAbsAdLoaderAdapter{
    public *;
    protected <fields>;
}
# csj sdk end


#baidu start
-keep class com.baidu.mobads.** { *; }

# ks SDK
-keep class org.chromium.** {*;}
-keep class org.chromium.** { *; }
-keep class aegon.chrome.** { *; }
-keep class com.kwai.**{ *; }
-dontwarn com.kwai.**
-dontwarn com.kwad.**
-dontwarn com.ksad.**
-dontwarn aegon.chrome.**

# GDT sdk start
-dontwarn com.qq.e.**
-keep class com.qq.e.** {*;}
-dontpreverify

# JD SDK
-keep class com.jd.ad.sdk.** { *; }

# Xm SDK
-keep class com.miui.zeus.** { *; }

# Oppo SDK
-keep class com.opos.** { *;}
-keep class com.heytap.msp.mobad.** { *;}
-keep class com.heytap.openid.** {*;}

-keep class okio.**{ *; }

-keeppackagenames com.heytap.nearx.tapplugin

# HuaWei SDK
-keep class com.huawei.openalliance.ad.** { *; }
-keep class com.huawei.hms.ads.** { *; }

# Vivo SDK
-dontwarn okhttp3.**
-dontwarn android.support.v4.**
-dontwarn com.squareup.okhttp.**
-keep class com.vivo.*.** { *; }
-dontwarn com.bytedance.article.common.nativecrash.NativeCrashInit
-keep class com.bytedance.sdk.openadsdk.** {*;}
-keep public interface com.bytedance.sdk.openadsdk.downloadnew.** {*;}
-keep class com.pgl.sys.ces.* {*;}
-keep class com.qq.e.** {
    public protected *;
}
-keep class android.support.v4.**{
    public *;
}
-dontwarn com.vivo.secboxsdk.**
-keep class com.vivo.secboxsdk.SecBoxCipherException { *; }
-keep class com.vivo.secboxsdk.jni.SecBoxNative { *; }
-keep class com.vivo.secboxsdk.BuildConfig { *; }

-keep class android.support.v7.widget.** {*;}
````
# 3 在flutter中展示广告
亿帆在flutter平台上暂时仅支持全屏开屏，激励视频，插屏广告，Banner广告（仅Android支持）。

在一个flutter engine实例中，存在一个亿帆广告实例，每个实例都要确保初始化之后再请求广告。如果有多个flutter engine实例，则每个实例中都要初始化万汇sdk。

## 3.1 初始化
我们一般在Widget的initState方法中做初始化工作，示例代码如下，
````
  import 'package:yf_ads/yf_ads.dart';

  class _MyAppState extends State<MyApp> {
    var reaperFlutterPlugin;
  
  @override
  void initState() {
    super.initState();
    reaperFlutterPlugin = ReaperFlutterPlugin.instance;
    initReaperSdk();
  }
  
  Future<void> initReaperSdk() async {
    var reaperInitSdkBaseBean;
    if (Platform.isAndroid) {
      const RELEASE_APP_ID = "24f8e0ba-697e-4ee5-b4fb-30cc1c9f2031";
      const RELEASE_APP_KEY = "24f8e0ba-697e-4ee5-b4fb-30cc1c9f2031";
      reaperInitSdkBaseBean = const YFAdsInitSDKAndroidBean(
          appId: RELEASE_APP_ID, appKey: RELEASE_APP_KEY, enableLog: true);
    }
    await reaperFlutterPlugin.initAdsSDK(reaperInitSdkBaseBean);
}
````
### 3.1.1 Android初始化
YFAdsInitSDKAndroidBean作为Android平台的初始化类，封装了如下参数：
````
    final String? appId;
    final String? appKey;
    
    final String? ext1;
    final String? ext2;
    final String? ext3;
    final String? ext4;
    final String? ext5;
    final String? ext6;
    final String? ext7;
    final String? ext8;
    
    final String? channelId;
    final bool? permissionLocation;
    final String? firstActiveTime;
    
    final List<double>? location;
    
    final bool? permissionPhoneState;
    final bool? permissionWifiState;
    final bool? permissionWriteExternal;
    final bool? permissionOaid;
    final bool? permissionAndroidId;
    final String? imei;
    final String? oaid;
    final bool? permissionAppList;
    final bool? enableLog;
````
参数说明如下：</br>
* 1、appId和appKey被标记为必须传递的参数；
* 2、ext1，ext2，ext3，ext4，ext5，ext6，ext7，ext8，非必须参数。设置扩展维度参数，需要接入方与亿帆SDK产品确认扩展维度参数的意义，报表数据可以根据这些扩展维度参数细分。如下，第一个参数设置为地区，第二个参数设置为电话号码，第三个参数设置为是否VIP，第四个参数设置为是否新手保护，如果接入方不需要扩展参数，可以忽略这8个方法；
* 3、channelId， 非必须参数，可以设置渠道号；
* 4、permissionLocation，permissionPhoneState，permissionWifiState，permissionWriteExternal，permissionOaid，permissionAppList，非必须参数。用于设置隐私信息权限开关，默认为true。
* 5、firstActiveTime，非必须参数。设置第一次激活时间。
* 6、location，非必须，如果permissionLocation传false，可以通过该参数传递具体的位置经纬度。
* 7、imei，非必须，如果permissionPhoneState传false，可以通过该参数传递imei值。
* 8、oaid，非必须，如果permissionOaid传false，可以通过该参数传递oaid值。
* 9、permissionAppList，非必须，获取应用列表权限。
* 10、enableLog，非必须，是否开启亿帆sdk日志。

## 3.2 展示插屏广告
插屏广告请求示例如下:
````
    Future<void> showInsertAd() async {
    var positionId;
    if (Platform.isAndroid) {
      positionId = "503e92d8-2e2a-472c-b045-0f0f0282830c";
    }
    var reaperInsertBean =YFAdsInteractionBean(
        positionId: positionId, callBack: insertAdCallBack);
    await reaperFlutterPlugin.loadInteractionAd(reaperInsertBean);
    }
    
    void insertAdCallBack(YFAdsCallBack data) {
        print("insertAdCallBack:${data.method!}");
    }
````

### 3.2.1 请求参数说明
YFAdsInteractionBean是插屏广告参数的封装类，封装参数如下：
````
    final String? positionId;
    final Function(YFAdsCallBack)? callBack;
````
* positionId，广告位，是必传参数。
* callBack，请求回调，用于接收插屏广告回调。

### 3.2.2 广告回调
如果在请求广告时，定义了回调函数，插件会将请求过程回调，回调数据为ReaperAdCallBack对象。

YFAdsCallBack对象中包含了回调函数名称，如果是报错相关的函数，会有报错信息回调，对于插屏广告，会有如下函数及相关参数回调：
````
    /**
     * 广告填充成功
     */
    void onAdSuccess();

    /**
     * 广告被关闭
     */
    void onAdClosed();

    /**
     * 广告被点击
     */
    void onAdClicked();

    /**
     * 广告曝光
     */
    void onAdShow();
    
    /**
     * 广告请求失败
     *
     * @param requestId 请求标识
     * @param errMsg    失败原因
     */
    void onFailed(Strig errMsg);
````

## 3.3 展示开屏广告
YFAdsSplashBean是开屏的广告参数封装类，封装参数如下：
````
final String? positionId;

///only for android platform usage,for next page after splash ad page finished
///if you set it,you should not set splashClassPath or this parameter will ignore.
final String? destActivityFullPath;

///only for android platform usage,you can define it for your own splash ad page
final String? splashClassPath;
````

* positionId，广告位，是必传参数。
* destActivityFullPath，仅适用于Android平台传递参数，用于指定开屏完成之后的目标页面路径（Activity类的全路径）。
* splashClassPath， 仅适用于Android平台传递参数，用于设置接入方的开屏Activity类全路径。

### 3.3.1 全屏默认背景
从Activity或UIViewController被拉起到开屏广告展示出来之前，广告需要请求才能加载。这段时间内，页面默认背景为白色。flutter插件提供了方法可以设置这段时间内的默认背景。

* Android
将默认背景图片重命名为default_splash_back.png，并将其复制到工程的assets文件夹中，随apk打包。

## 3.4 展示激励视频
激励视频广告请求示例如下：
````
  Future<void> showRewardVideoAd() async {
    var reaperRewardVideoBean;
    var positionId;
    if (Platform.isAndroid) {
      positionId = "e46b3594-f3db-4f8d-b602-ec429bb37fd2";
    } else if (Platform.isIOS) {
      positionId = "c1d463494c01418e8a31e36398f37a82";
    }

    ///object will send to your server to verify reward
    ///adCode means position id which will be replaced with real position id in reaper sdk,and should be fixed with %%POS_ID%%
    ///adSrc means ad source which will be replaced with real ad source in reaper sdk,and should be fixed with %%AD_SRC%%
    ///other parameters can be defined dynamic by yourself.
    var reaperRewardVideoExtraBean = const YFAdsRewardVideoExtraBean(
        uid: "1156642",
        rewardVideoType: 2,
        taskId: "70",
        taskAction: "rewardVideo4energy",
        version: "3.42.0_debug",
        adCode: "%%POS_ID%%",
        adSrc: "%%AD_SRC%%");
    var extra = jsonEncode(reaperRewardVideoExtraBean);
    reaperRewardVideoBean = YFAdsRewardVideoBean(
        positionId: positionId, extra: extra, callBack: rewardVideoAdCallBack);
    await reaperFlutterPlugin.loadRewardVideoAd(reaperRewardVideoBean);
  }

  void rewardVideoAdCallBack(
      YFAdsRewardVideoCallBack reaperRewardVideoAdCallBack) {
    print("rewardVideoAdCallBack:" + reaperRewardVideoAdCallBack.method!);
    if (reaperRewardVideoAdCallBack.method ==
        YFAdsRewardVideoCallBack.YF_ADS_ON_REWARD_VERIFY) {
      print(
          "rewardVideoAdCallBack rewardAmount is ${reaperRewardVideoAdCallBack.rewardAmount},"
          "rewardName is ${reaperRewardVideoAdCallBack.rewardName},"
          "rewardVerify is ${reaperRewardVideoAdCallBack.rewardVerify}");
    }
  }
````
### 3.4.1 请求参数说明
YFAdsRewardVideoBean是激励视频广告参数的封装类，封装参数如下：
````
  static const REWARD_REWARD_VIDEO_VERTICAL = 1;
  static const REWARD_REWARD_VIDEO_HORIZONTAL = 2;

  final String? rewardName;
  final String? userId;
  final int? rewardAmount;
  final String? extra;
  final int? orientation;
  final String? positionId;
  final Function(YFAdsRewardVideoCallBack)? callBack;
````
* positionId，广告位，是必传参数。
* callBack，请求回调。
* rewardName和rewardAmount是iOS平台的非必须参数，用于设置激励名称和数量，Android平台不需要这两个参数。
* orientation，默认为竖屏。暂时支持横屏
* userId和extra，激励视频奖励回调验证参数，如果需要在您的后台验证激励，则必须同时传递这两个参数，如果不验证，则可以不传。穿山甲和广点通广告源支持服务端验证，支持2个透传参数：userId、extra，激励视频达成之后，第三方广告源会回调接入方的服务端地址，而且自动带上这两个透传参数。
* extra参数使用ReaperRewardVideoExtraBean对象封装，接入方可以定制成自己需要的参数。这些参数中，如果接入方服务端需要广告位和广告源参数，则必须确保adCode和adSrc的key和value不能变，必须恒定为adCode: "%%POS_ID%%"和adSrc: "%%AD_SRC%%"。

### 3.4.2 广告回调
如果在请求广告时，定义了回调函数，插件会将请求和填充过程回调，回调数据为YFAdsRewardVideoCallBack对象。</br>
对象中包含了回调函数名称，如果是报错相关的函数，会有报错信息回调，对于激励视频广告，会有如下函数及相关参数回调，如果需要确认奖励是否发放，则需要重点关注onRewardVerify（Android）方法的。详细回调方法如下：
````
    /**
     * 视频广告加载后，视频资源缓存到本地的回调，在此回调后，播放本地视频，流畅不阻塞
     */
    void onRewardVideoCached();
    
    /**
     * 广告播放完成
     */
    void onVideoComplete();
    
    /**
     * 跳过视频播放
     */
    void onSkippedVideo();
    
    /**
     *  视频播放完成后，奖励验证回调
     */
    void onRewardServerInf(String reward_name, String reward_verify);
    
    /**
     * 跳过视频播放
     */
    void onSkippedVideo();
    
    /**
     * 广告填充完成
     */
    void onAdSuccess();
    
    /**
     * 广告曝光
     */
    void onAdShow();
    
    /**
     * 广告点击
     */
    void onAdClicked();
    
    /**
     * 广告关闭
     */
    void onAdClosed();
  
    /**
     * 广告失败
     */
    void onFailed();

````

## 3.5 展示Banner广告

Banner广告请求示例如下：

````
class YFAdsBannerViewDemo extends StatefulWidget {
    @override
    State<StatefulWidget> createState() {
        return YFAdsBannerViewDemoState();
      }
    }

    class YFAdsBannerViewDemoState extends State<YFAdsBannerViewDemo> {
      bool showBanner = true;
    
      void bannerViewCallBack(YFAdsCallBack args) {
        print("bannerViewCallBack is ${args.method}");
        if (args.method == "onDislike" || args.method == "WHNewBannerAdClosed") {
          setState(() {
            showBanner = false;
          });
        }
    }

    void reloadBannerAd() {
        print("reloadBannerAd");
        YFAds.instance.clearBannerPageCache();
        setState(() {
          showBanner = true;
        });
    }

    @override
    Widget build(BuildContext context) {
        print("banner view demo build");
        var positionId;
        var androidOS = false;
        if (defaultTargetPlatform == TargetPlatform.android) {
          positionId = "b70687e8-1eb8-4615-88df-65858fe70842";
          androidOS = true;
        }
    
        var reloadBannerView = Align(
            alignment: Alignment.center,
            child: ElevatedButton(
              onPressed: reloadBannerAd,
              child: const Text("加载banner广告"),
            ));
    
        var bannerAdView = showBanner
            ? YFAdsBannerView(
                // key: Key("$bannerCornerSize"),
                onAdCallBack: bannerViewCallBack,
                topMarginSize: 36,
                positionId: positionId,
                cornerSize: 30,
                adSizeType: androidOS
                    ? AndroidBannerAdType.AD_SIZE_W600xH400
                    : IOSBannerAdType.AD_SIZE_NEW_H800,
              ): reloadBannerView;
              
        return Scaffold(
          appBar: AppBar(
            title: const Text('Banner广告'),
          ),
          body: bannerAdView,
        );
    }

    @override
    void dispose() {
        super.dispose();
        print("YFAdsBannerViewDemo dispose");
        YFAds.instance.clearBannerPageCache();
    }
}
````

YFAds.instance.clearBannerPageCache(); 该方法必须调用，建议在页面关闭回调里面调用。

### 3.5.1 请求参数说明

YFAdsBannerView是Banner广告参数的封装类，封装参数如下：

````
  final int? uniqueId;
  const YFAdsBannerView(
      {Key? key,
      @required this.adSizeType,
      this.marginSize = 12,
      this.cornerSize = 0,
      this.topMarginSize = 0,
      this.onAdCallBack,
      this.uniqueId = -1,
      @required this.positionId,
      this.showDislike = true,
      this.aqyAd = false,
      this.expressAd = false,
      this.heightOffset = 0,
      this.aliAd = false})
      : super(key: key);
````
* adSize，是AdSize对象，里面包含了不同banner广告尺寸，说明如下：
````
    AD_SIZE_W690xH388("W690xH388"), // 1.778，宽高比
    AD_SIZE_W600xH400("W600xH400"), // 1.5
    AD_SIZE_W600xH300("W600xH300"), // 2
    AD_SIZE_W600xH260("W600xH260"), // 2.307
    AD_SIZE_W600xH260_L_TEXT_R_PICTURE("W600XH260左文右图"), // 2.307
    AD_SIZE_W600xH150("W600xH150"), // 4
    AD_SIZE_W600xH150_L_TEXT_R_PICTURE("W600xH150左文右图"), // 4
    AD_SIZE_W640xH100("W640xH100"), // 6.4
    AD_SIZE_W640xH100_L_TEXT_R_PICTURE("W640xH100左文右图"), // 6.4
    AD_SIZE_W600xH90("W600xH90"),  // 6.667
    AD_SIZE_W600xH90_L_TEXT_R_PICTURE("W600xH90左文右图"),  // 6.667
    AD_SIZE_W300xH200("W300xH200"),
    AD_SIZE_W300xH200_BLACK("W300xH200透明背景"),
    AD_SIZE_W600xH540("W600xH540"),
    AD_SIZE_W600xH540_TRANSPARENT("W600xH540透明背景"),
    AD_SIZE_W16H9_B52("宽高16:9，底部多52"),
    AD_SIZE_W16H9_B34("宽高16:9，底部多34"),
    AD_SIZE_W16H9("宽高16:9")
````
* marginSize，banner广告与父视图左右两边的margin，默认为12dp；
* cornerSize，背景圆角大小，默认为0；
* topMarginSize，与顶部的 margin尺寸；
* uniqueId，view id，如果只是展示单个banner item，不需要传该参数；如果是列表展示多个banner item，该参数必须传，且每个item的值必须唯一；
* positionId，广告位，必须参数；
* showDislike，是否展示广告关闭按钮，默认展示；
* heightOffset，高度偏移，如果高度不符合需求，可以针对性设置偏移量；
* onAdCallBack，请求回调，用于接收Banner广告回调。

### 3.5.2 广告回调
如果在请求广告时，定义了回调函数，插件会将请求过程回调，回调数据为YFAdCallBack对象。

YFAdCallBack对象中包含了回调函数名称，如果是报错相关的函数，会有报错信息回调，对于Banner广告，会有如下函数及相关参数回调：

````
  // 广告填充成功
  void onBannerPositionAdLoaded(int view_id);
  
  // 广告曝光成功
  void onAdShow(int view_id);
  
  // 广告点击
  void onAdClicked(int view_id);
  
  // 广告关闭 
  void onDislike(int view_id);
 
  // 广告加载失败
  void onFailed(String errorMsg, int view_id)

````

## 3.6 列表中展示Banner广告(限Android)

列表Banner广告请求示例如下：
````
  class YFAdsBannerListViewDemo extends StatefulWidget {
    @override
    State<StatefulWidget> createState() {
      return YFAdsBannerListViewDemoState();
    }
  }

  class YFAdsBannerListViewDemoState extends State<YFAdsBannerListViewDemo> {
    var deletedBannerItem = [];

    void bannerListViewItemCallBack(YFAdsCallBack reaperAdCallBack) {
      print(
        "bannerListViewItemCallBack is ${reaperAdCallBack.method} and viewId is ${reaperAdCallBack.viewId}");
      if (reaperAdCallBack.method == "onDislike") {
        setState(() {
        deletedBannerItem.add(reaperAdCallBack.viewId);
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    var positionId;
    var androidOS = false;
    if (defaultTargetPlatform == TargetPlatform.android) {
      positionId = "b70687e8-1eb8-4615-88df-65858fe70842";
      androidOS = true;
    }
    return Scaffold(
      appBar: AppBar(
        title: const Text('Banner列表广告'),
      ),
      body: ListView.builder(
        itemBuilder: (BuildContext context, int index) {
          if (kDebugMode) {
            print("YFAdsBannerListViewDemo index is $index");
          }
          if (index % 5 != 0) {
            return Container(
              width: 200,
              height: 50,
              child: Text(
                "我是第$index个item",
                style: TextStyle(fontSize: 18),
              ),
            );
          }
          bool delete = deletedBannerItem.contains(index);
          return delete
              ? Container()
              : YFAdsBannerView(
                  uniqueId: index,
                  onAdCallBack: bannerListViewItemCallBack,
                  topMarginSize: 36,
                  positionId: positionId,
                  cornerSize: 30,
                  showDislike: true,
                  adSizeType: androidOS
                      ? AndroidBannerAdType.AD_SIZE_W600xH400
                      : IOSBannerAdType.AD_SIZE_NEW_H800,
                );
        },
        itemCount: 18,
      ),
    );
  }

  @override
  void dispose() {
    super.dispose();
    print("YFAdsBannerListViewDemo dispose");
    YFAds.instance.clearBannerPageCache();
  }
}
````
YFAds.instance.clearBannerPageCache(); 该方法必须调用，建议在页面关闭回调里面调用。</br>

列表Banner广告与单个Banner广告在请求方式上基本无差异，需要接入方额外处理banner item被关闭后的处理，在demo中已经有示例。</br>

需要特别注意的是，在构造YFAdsBannerView对象时，列表Banner必须传递uniqueId参数，且每个item持有的uniqueId不能相等。</br>

## 3.7 展示信息流广告

信息流广告请求示例如下：

````
  class YFAdsExpressFeedViewDemo extends StatefulWidget {
    @override
    State<StatefulWidget> createState() {
      return YFAdsExpressFeedViewDemoState();
    }
  }

  class YFAdsExpressFeedViewDemoState extends State<YFAdsExpressFeedViewDemo> {
    bool showExpressFeedAd = true;

    void expressFeedViewCallBack(YFAdsExpressFeedCallBack args) {
      if (!mounted) {
        return;
      }
      print("expressFeedViewCallBack is ${args.method}");
      if (args.method == "onDislike") {
        setState(() {
          showExpressFeedAd = false;
        });
     }
   }

  void reloadExpressFeedAd() {
    print("reloadExpressFeedAd");
    YFAds.instance.clearExpressFeedPageCache();
    setState(() {
      showExpressFeedAd = true;
    });
  }

  @override
  Widget build(BuildContext context) {
    print("express feed view demo build");
    var positionId;
    if (defaultTargetPlatform == TargetPlatform.android) {
      positionId = "0a29f771-de25-41d6-90e7-1269a6a9d85a";
    }

    var reloadBannerView = Align(
        alignment: Alignment.center,
        child: ElevatedButton(
          onPressed: reloadExpressFeedAd,
          child: const Text("加载信息流广告"),
        ));

    var expressFeedAdView = showExpressFeedAd
        ? YFAsdExpressFeedView(
            onAdCallBack: expressFeedViewCallBack,
            positionId: positionId,
          )
        : reloadBannerView;
    return Scaffold(
        appBar: AppBar(
          title: const Text('信息流广告'),
        ),
        body: Container(
          margin: const EdgeInsets.only(top: 24),
          child: expressFeedAdView,
        ));
  }

  @override
  void dispose() {
    super.dispose();
    print("YFAdsExpressFeedViewDemo dispose");
    YFAds.instance.clearExpressFeedPageCache();
  }
}
````
YFAds.instance.clearExpressFeedPageCache();该方法必须调用，建议在页面关闭回调里面调用。</br>

### 3.7.1 请求参数说明

YFAsdExpressFeedView是信息流广告参数的封装类，封装参数如下：

````
  final String? positionId;
  final Function(YFAdsExpressFeedCallBack)? onAdCallBack;

  ///if you want to load several banner ad in list view,
  ///you must set uniqueId parameter with unique value for every item
  final int? uniqueId;

  const YFAsdExpressFeedView({
    Key? key,
    this.onAdCallBack,
    this.uniqueId = -1,
    this.width,
    this.height,
    @required this.positionId,
  }) : super(key: key);
````
* uniqueId，view id，如果只是展示单个信息流item，不需要传该参数；如果是列表展示多个信息流 item，该参数必须传，且每个item的值必须唯一；
* positionId，广告位，必须参数；
* onAdCallBack，请求回调，用于接收信息流广告回调。

### 3.7.2 返回回调说明

YFAdsExpressFeedCallBack对象中包含了回调函数名称，如果是报错相关的函数，会有报错信息回调，对于信息流广告，会有如下函数及相关参数回调：

````
  // 填充成功
  void onNativeExpressAdLoaded(int view_id);
  
  // 渲染失败
  void onRenderFail(int view_id);
  
  // 加载失败
  void onFailed(int view_id);
  
  // 曝光成功
  void onAdShow(int view_id);
  
  // 广告点击
  void onAdClicked(int view_id);
  
  // 广告关闭
  void onDislike(int view_id);
  
  // 更新视图大小：view的ID，实际高度，类型
  void onContainerSizeChanged(int view_id, int container_height, String ad_type);

````

## 3.8 展示信息流列表广告

信息流列表请求广告的代码如下：

````
  class ExpressFeedListViewDemo extends StatefulWidget {
    const ExpressFeedListViewDemo({Key? key}) : super(key: key);

    @override
    State<StatefulWidget> createState() {
      return YFAdsExpressFeedViewDemoState();
    }
  }

  class YFAdsExpressFeedViewDemoState extends State<ExpressFeedListViewDemo> {
    var deletedExpressFeedItem = [];
    ///先设置默认高度，当然也可以设置成其他高度已知的默认Widget代替
    var defaultHeight = 350.0;
    ///模板信息流广告的高度在填充之前未知，在添加到渲染列表之后，才能得到View的高度
    ///所以这里会有一个map维护每个item的高度，先设置一个默认高度，待确认高度之后动态调整
    var heightMap = Map<int, double>();

    var defaultSize = 18;

    void expressFeedListViewItemCallBack(
      YFAdsExpressFeedCallBack reaperAdCallBack) {
      print(
        "expressFeedListViewItemCallBack is ${reaperAdCallBack.method} and viewId is ${reaperAdCallBack.viewId}");

      if (reaperAdCallBack.method == "onContainerSizeChanged") {
        var tempHeight = reaperAdCallBack.containerHeight!;
        print("expressFeedListViewItemCallBack tempHeight is $tempHeight");
        if (tempHeight > 1) {
          setState(() {
            heightMap[reaperAdCallBack.viewId!] = tempHeight.toDouble();
        });
      }
      return;
    }
    if (reaperAdCallBack.method == "onDislike" ||
        reaperAdCallBack.method == "onFailed" ||
        reaperAdCallBack.method == "WHNewExpressFeedAdClosed") {
      setState(() {
        deletedExpressFeedItem.add(reaperAdCallBack.viewId);
      });
    }
  }

  @override
  void initState() {
    super.initState();
    for (var i = 0; i < defaultSize; i++) {
      heightMap[i] = defaultHeight;
    }
  }

  @override
  Widget build(BuildContext context) {
    var positionId;
    if (defaultTargetPlatform == TargetPlatform.android) {
      positionId = "0a29f771-de25-41d6-90e7-1269a6a9d85a";
    }
    double width = MediaQuery.of(context).size.width;
    double height = 0;
    return Scaffold(
      appBar: AppBar(
        title: const Text('信息流列表广告'),
      ),
      body: ListView.builder(
        itemBuilder: (BuildContext context, int index) {
          print("ExpressFeedListViewDemo index is $index");
          if (index % 5 != 0) {
            return Container(
              width: width,
              height: 50,
              child: Text(
                "我是第$index个item",
                style: TextStyle(fontSize: 18),
              ),
            );
          }
          bool delete = deletedExpressFeedItem.contains(index);
          return delete
              ? Container()
              : Container(
                  width: width,
                  height: heightMap[index],
                  child: YFAsdExpressFeedView(
                    width: width,
                    height: height,
                    uniqueId: index,
                    onAdCallBack: expressFeedListViewItemCallBack,
                    positionId: positionId,
                  ));
        },
        itemCount: 18,
      ),
    );
  }

  @override
  void dispose() {
    super.dispose();
    print("ExpressFeedListViewDemo dispose");
    YFAds.instance.clearExpressFeedPageCache();
  }
}

````

YFAds.instance.clearExpressFeedPageCache(); 该方法必须调用，建议在页面关闭回调里面调用。</br>

列表信息流广告与单个信息流广告在请求方式上基本无差异，需要接入方额外处理信息流 item被关闭后的处理，在demo中已经有示例。</br>

需要特别注意的是，在构造YFAsdExpressFeedView对象时，列表信息流必须传递uniqueId参数，且每个item持有的uniqueId不能相等。</br>

## 4. 广告回调
### 4.1 YFAdCallBack
````
class YFAdCallBack {
    ///回调的方法
    final String? method;
    ///错误方法回调会有错误详情
    final String? error;
    ///viewid,针对banner 列表广告可以针对该参数区分item
    final int? viewId;
    //回调广告类型
    final String? adType;
}
````
* adType 广告类型枚举如下：
````
  static const AD_TYPE_BANNER = "type_banner";
  static const AD_TYPE_INTERACTION = "type_interaction";
  static const AD_TYPE_REWARD_VIDEO = "type_reward_video";
````

### 4.2 YFAdsRewardVideoCallBack
用于激励视频广告回调，继承自YFAdCallBack，也包含ReaperAdCallBack的参数。

````
class YFRewardVideoAdCallBack extends ReaperAdCallBack {
    ///是否发放奖励，true,是;false,否
    final bool? rewardVerify;
    ///奖励名称
    final String? rewardName;
}
````