import 'dart:core';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:flutter/foundation.dart';
import 'package:yf_ads/ads/yf_ads_express_feed_view.dart';
import 'package:yf_ads/callbacks/yf_ads_express_feed_callback.dart';
import 'package:yf_ads/yf_ads.dart';

class ExpressFeedListViewDemo extends StatefulWidget {
  const ExpressFeedListViewDemo({Key? key}) : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return YFAdsExpressFeedViewDemoState();
  }
}

class YFAdsExpressFeedViewDemoState extends State<ExpressFeedListViewDemo> {
  var deletedExpressFeedItem = [];
  ///先设置默认高度，当然也可以设置成其他高度已知的默认Widget代替
  var defaultHeight = 750.0;
  ///模板信息流广告的高度在填充之前未知，在添加到渲染列表之后，才能得到View的高度
  ///所以这里会有一个map维护每个item的高度，先设置一个默认高度，待确认高度之后动态调整
  var heightMap = Map<int, double>();

  var defaultSize = 18;

  void expressFeedListViewItemCallBack(
      YFAdsExpressFeedCallBack reaperAdCallBack) {
    print(
        "expressFeedListViewItemCallBack is ${reaperAdCallBack.method} and viewId is ${reaperAdCallBack.viewId}");

    if (reaperAdCallBack.method == "onContainerSizeChanged") {
      var tempHeight = reaperAdCallBack.containerHeight!;
      print("expressFeedListViewItemCallBack tempHeight is $tempHeight");
      if (tempHeight > 1) {
        setState(() {
          heightMap[reaperAdCallBack.viewId!] = tempHeight.toDouble();
        });
      }
      return;
    }
    if (reaperAdCallBack.method == "onDislike" ||
        reaperAdCallBack.method == "onFailed" ||
        reaperAdCallBack.method == "WHNewExpressFeedAdClosed") {
      setState(() {
        deletedExpressFeedItem.add(reaperAdCallBack.viewId);
      });
    }
  }

  @override
  void initState() {
    super.initState();
    for (var i = 0; i < defaultSize; i++) {
      heightMap[i] = defaultHeight;
    }
  }

  @override
  Widget build(BuildContext context) {
    var positionId;
    if (defaultTargetPlatform == TargetPlatform.android) {
      positionId = "0a29f771-de25-41d6-90e7-1269a6a9d85a";
      // jd
      // positionId = "08f3a964-1dae-4193-9f5a-ba533fc50f39";
    } else if (defaultTargetPlatform == TargetPlatform.iOS) {
      positionId = "ae1caa917bbf483aad9127b613668cac";
    }
    double width = MediaQuery.of(context).size.width;
    double height = 0;
    return Scaffold(
      appBar: AppBar(
        title: const Text('信息流列表广告'),
      ),
      body: ListView.builder(
        itemBuilder: (BuildContext context, int index) {
          print("ExpressFeedListViewDemo index is $index");
          if (index % 5 != 0) {
            return Container(
              width: width,
              height: 50,
              child: Text(
                "我是第$index个item",
                style: TextStyle(fontSize: 18),
              ),
            );
          }
          bool delete = deletedExpressFeedItem.contains(index);
          return delete
              ? Container()
              : Container(
                  width: width,
                  height: heightMap[index],
                  child: YFAsdExpressFeedView(
                    uniqueId: index,
                    onAdCallBack: expressFeedListViewItemCallBack,
                    positionId: positionId,
                  ));
        },
        itemCount: 18,
      ),
    );
  }

  @override
  void dispose() {
    super.dispose();
    print("ExpressFeedListViewDemo dispose");
    YFAds.instance.clearExpressFeedPageCache();
  }
}
