//
//  WHVideoPathAdManager.m
//  BURelyFoundation
//
//  Created by os on 2022/4/18.
//

#import <Foundation/Foundation.h>
#import "YFAdsConstant.h"
#import <Flutter/Flutter.h>
#import "YFAdsVideoManager.h"
#import "YFAdsUtils.h"
#import <YFAdsSDK/YFAdPatch.h>

@interface YFAdsVideoManager ()<YFAdPatchDelegate>

@property (nonatomic, strong) FlutterEventSink flutterEventSink;
@property (nonatomic, strong) UIViewController * viewController;
@property (nonatomic, strong) YFAdPatch *adPatch;
@property (nonatomic, strong) UIView *contentV;
@property (nonatomic, strong) YFAdPatch *videoPatchAd;
//@property (nonatomic, strong) WHVideoPatch *videoPatch;
@property (nonatomic, strong) NSNumber *currentUniqueKey;
@property (nonatomic, assign) float frameX;
@property (nonatomic, assign) float frameY;
@property (nonatomic, assign) float margin;

@end

@implementation YFAdsVideoManager

- (instancetype)initWithParams:(FlutterEventSink)flutterEventSink viewController:(UIViewController *)viewController{
    self = [super init];
    _flutterEventSink = flutterEventSink;
    _viewController = viewController;
    return self;
}

- (void)showAd:(NSDictionary *)parametersDic {
    NSString *positionId = parametersDic[KEY_POSITION_ID];
    _currentUniqueKey = parametersDic[KEY_UNIQUE_KEY];
    NSLog(@"showAd currentUniqueKey is %@",_currentUniqueKey);

    float defaultMargin = 60;
    NSNumber *configMargin = parametersDic[KEY_VIDEO_PATH_MARGIN];
    if(configMargin && configMargin.floatValue > 0){
        defaultMargin = configMargin.floatValue;
    }
    self.margin = defaultMargin;

    float defaultFrameX = 0;
    NSNumber *configFrameX = parametersDic[KEY_VIDEO_PATH_FRAME_X];
    if([configFrameX isKindOfClass:NSNumber.class] && configFrameX.floatValue > 0){
        defaultFrameX = configFrameX.floatValue;
    }

    float defaultFrameY = 0;
    NSNumber *configFrameY = parametersDic[KEY_VIDEO_PATH_FRAME_Y];
    if([configFrameY isKindOfClass:NSNumber.class]  && configFrameY.floatValue > 0){
        defaultFrameY = configFrameY.floatValue;
    }
    self.frameX = defaultFrameX;
    self.frameY = defaultFrameY;

    self.contentV = [[UIView alloc] initWithFrame:CGRectZero];
    self.adPatch = [[YFAdPatch alloc] initWithAdUnitID:positionId adContainer:_contentV viewController:self.viewController];
    self.adPatch.delegate = self;
    [self.adPatch loadAndShowAd];
 

}

//- (void)WHVideoPatchDidPlayFinish:(WHVideoPatch *_Nullable)videoPatch {
//    NSLog(@"WH视频贴片广告播放完成");
//    NSMutableDictionary *result = [NSMutableDictionary new];
//    [result setValue:@"WHVideoPatchAdDidPlayFinish" forKey:KEY_METHOD];
//    [self callBack:result];
//}

-(void)callBack:(NSMutableDictionary *)resultDic{
    [resultDic setValue:AD_TYPE_VIDEO_PATH forKey:KEY_AD_TYPE];
    [resultDic setValue:_currentUniqueKey forKey:KEY_VIEW_ID];
    NSString * result = [YFAdsUtils dictionary2JsonString:resultDic];
    _flutterEventSink(result);
}


- (void)deallocAd {
    [self.contentV removeFromSuperview];
    self.contentV = nil;
    if (self.adPatch) {
        self.adPatch.delegate = nil;
        self.adPatch = nil;
    }
}

/// 广告数据加载成功
- (void)fcAdLoadSuccess:(YFAdBaseAdapter *)model {
    NSLog(@"广告加载成功 %s", __func__);
    
    if (model) {
        CGFloat width = [UIScreen mainScreen].bounds.size.width - self.margin;
        CGFloat height = width * 9.0 / 16;
        self.contentV.frame = CGRectMake(self.frameX, self.frameY, width, height);

        NSMutableDictionary *result = [NSMutableDictionary new];
        [result setValue:@"yfVideoPatchAdDidLoad" forKey:KEY_METHOD];
        [self callBack:result];
        if([self.flutterVideoPathViewDelegate respondsToSelector:@selector(yfVideoPathViewLoadSuccess:)]){
            [self.flutterVideoPathViewDelegate yfVideoPathViewLoadSuccess:self.contentV];
        }
    }

}

/// 广告加载失败
- (void)fcAdFailedWithError:(NSError *)error description:(NSDictionary *)description{
    NSLog(@"贴片广告展示失败 %s  error: %@ 详情:%@", __func__, error, description);
    NSMutableDictionary *result = [NSMutableDictionary new];
    [result setValue:@"yfVideoPathAdLoadFail" forKey:KEY_METHOD];
    [self callBack:result];
    [self deallocAd];
}


/// 广告曝光
-(void)fcAdExposured:(id)adapter{
    NSLog(@"贴片广告曝光回调 %s", __func__);
    NSMutableDictionary *result = [NSMutableDictionary new];
    [result setValue:@"yfVideoPatchAdDidShow" forKey:KEY_METHOD];
    [self callBack:result];
}

/// 广告点击
- (void)fcAdClicked:(id)adapter{
    NSLog(@"贴片广告点击 %s", __func__);
    NSMutableDictionary *result = [NSMutableDictionary new];
    [result setValue:@"yfVideoPatchAdDidClick" forKey:KEY_METHOD];
    [self callBack:result];
}

/// 广告关闭回调
- (void)fcAdDidClose:(id)adapter{
    NSLog(@"贴片广告关闭了 %s", __func__);
    [self deallocAd];
}

-(void)dealloc {
    NSLog(@"贴片释放 %s", __func__);
    [self deallocAd];
}

@end
