
#import <Foundation/Foundation.h>
#import "YFAdsBannerViewFactory.h"
#import "YFAdsUtils.h"
#import "YFAdsBannerView.h"
#import "YFAdsConstant.h"


@interface YFAdsBannerViewFactory ()
@property (nonatomic, strong) FlutterEventSink flutterEventSink;
@property (nonatomic, strong) UIViewController *viewController;
@property (nonatomic, strong) NSMutableDictionary *bannerViewCachedDic;

@end

@implementation YFAdsBannerViewFactory

- (instancetype)initViewFactory {
    self = [super init];
    if (self) {
        _bannerViewCachedDic = [NSMutableDictionary new];
    }
    return self;
}

- (void)setFlutterEventSink:(FlutterEventSink)flutterEventSink{
    _flutterEventSink = flutterEventSink;
}

- (void)setFlutterViewController:(UIViewController *)viewController{
    _viewController = viewController;
}

- (NSObject<FlutterPlatformView>*)createWithFrame:(CGRect)frame
                                   viewIdentifier:(int64_t)viewId
                                        arguments:(id _Nullable)args {
    NSDictionary *parameters = [YFAdsUtils jsonString2Dictionary:args];
    if(!parameters){
        YFAdsBannerView *bannerView = [[YFAdsBannerView alloc] initWithFrame:frame arguments:args eventSink:_flutterEventSink viewController:_viewController];
        return bannerView;
    }
    NSNumber *uniqueId = parameters[KEY_UNIQUE_KEY];
    NSLog(@"bannerViewFactory createWithFrame uniqueId %@",uniqueId);
    if(_bannerViewCachedDic[uniqueId]){
        NSLog(@"bannerViewFactory createWithFrame hit the cache view and uniqueId %@",uniqueId);
        return _bannerViewCachedDic[uniqueId];
    }
    YFAdsBannerView *bannerView = [[YFAdsBannerView alloc] initWithFrame:frame arguments:parameters eventSink:_flutterEventSink viewController:_viewController];
    [_bannerViewCachedDic setValue:bannerView forKey:uniqueId];
    return bannerView;

}


- (NSObject<FlutterMessageCodec>*)createArgsCodec{
    return [FlutterStandardMessageCodec sharedInstance];
}

- (void)bannerViewPageFinished{
    if(_bannerViewCachedDic){
        NSLog(@"BannerViewFactory bannerViewPageFinished");
        [_bannerViewCachedDic removeAllObjects];
    }
}

@end
