//
//  NativeViewFactory.m
//  BURelyFoundation
//
//  Created by os on 2022/4/7.
//

#import <Foundation/Foundation.h>
#import "YFAdsNativeViewFactory.h"
#import "YFAdsUtils.h"
#import "YFAdsNativeView.h"
#import "YFAdsConstant.h"


@interface YFAdsNativeViewFactory ()
@property (nonatomic, strong) FlutterEventSink flutterEventSink;
@property (nonatomic, strong) UIViewController *viewController;
@property (nonatomic, strong) NSMutableDictionary *nativeViewCachedDic;
@property (nonatomic, strong) YFAdsNativeView *nativeView;

@end

@implementation YFAdsNativeViewFactory
//
- (instancetype)initViewFactory {
    self = [super init];
    if (self) {
        _nativeViewCachedDic = [NSMutableDictionary new];
    }
    return self;
}

- (void)setFlutterEventSink:(FlutterEventSink)flutterEventSink{
    _flutterEventSink = flutterEventSink;
}

- (void)setFlutterViewController:(UIViewController *)viewController{
    _viewController = viewController;
}

- (NSObject<FlutterPlatformView>*)createWithFrame:(CGRect)frame
                                   viewIdentifier:(int64_t)viewId
                                        arguments:(id _Nullable)args {
    NSDictionary *parameters = [YFAdsUtils jsonString2Dictionary:args];
    if(!parameters){
        _nativeView = [[YFAdsNativeView alloc] initWithFrame:frame arguments:args eventSink:_flutterEventSink viewController:_viewController];
        return _nativeView;
    }
    NSNumber *uniqueId = parameters[KEY_UNIQUE_KEY];
    NSLog(@"NativeViewFactory createWithFrame uniqueId %@",uniqueId);
    if(_nativeViewCachedDic[uniqueId]){
        NSLog(@"NativeViewFactory createWithFrame hit the cache view and uniqueId %@",uniqueId);
        return _nativeViewCachedDic[uniqueId];
    }
    _nativeView = [[YFAdsNativeView alloc] initWithFrame:frame arguments:parameters eventSink:_flutterEventSink viewController:_viewController];
    [_nativeViewCachedDic setValue:_nativeView forKey:uniqueId];
    return _nativeView;
}


- (NSObject<FlutterMessageCodec>*)createArgsCodec{
    return [FlutterStandardMessageCodec sharedInstance];
}

- (void)nativeViewPageFinished{
    NSLog(@"NativeViewFactory nativeViewPageFinished");
    if(_nativeViewCachedDic){
        [_nativeViewCachedDic removeAllObjects];
    }
    if(_nativeView){
        [_nativeView releaseNativeView];
    }
}

@end
