//
//  VideoPathViewFactory.m
//  BURelyFoundation
//
//  Created by os on 2022/4/19.
//

#import <Foundation/Foundation.h>
#import "YFAdsVideoViewFactory.h"
#import "YFAdsUtils.h"
#import "YFAdsVideoView.h"
#import "YFAdsConstant.h"


@interface YFAdsVideoViewFactory ()

@property (nonatomic, strong) FlutterEventSink flutterEventSink;
@property (nonatomic, strong) UIViewController *viewController;
@property (nonatomic, strong) NSMutableDictionary *videoPathViewCachedDic;

@end

@implementation YFAdsVideoViewFactory

- (instancetype)initViewFactory {
    self = [super init];
    if (self) {
        _videoPathViewCachedDic = [NSMutableDictionary new];
    }
    return self;
}

- (void)setFlutterEventSink:(FlutterEventSink)flutterEventSink{
    _flutterEventSink = flutterEventSink;
}

- (void)setFlutterViewController:(UIViewController *)viewController{
    _viewController = viewController;
}

- (NSObject<FlutterPlatformView>*)createWithFrame:(CGRect)frame
                                   viewIdentifier:(int64_t)viewId
                                        arguments:(id _Nullable)args {
    NSDictionary *parameters = [YFAdsUtils jsonString2Dictionary:args];
    if(!parameters){
        YFAdsVideoView *videoPathView = [[YFAdsVideoView alloc] initWithFrame:frame arguments:args eventSink:_flutterEventSink viewController:_viewController];
        return videoPathView;
    }
    NSNumber *uniqueId = parameters[KEY_UNIQUE_KEY];
    NSLog(@"VideoPathViewFactory createWithFrame uniqueId %@",uniqueId);
    if(_videoPathViewCachedDic[uniqueId]){
        NSLog(@"VideoPathViewFactory createWithFrame hit the cache view and uniqueId %@",uniqueId);
        return _videoPathViewCachedDic[uniqueId];
    }
    YFAdsVideoView *videoPathView = [[YFAdsVideoView alloc] initWithFrame:frame arguments:parameters eventSink:_flutterEventSink viewController:_viewController];
    [_videoPathViewCachedDic setValue:videoPathView forKey:uniqueId];
    return videoPathView;
}


- (NSObject<FlutterMessageCodec>*)createArgsCodec{
    return [FlutterStandardMessageCodec sharedInstance];
}

- (void)videoPathViewPageFinished{
    if(_videoPathViewCachedDic){
        NSLog(@"VideoPathViewFactory videoPathViewPageFinished");
        [_videoPathViewCachedDic removeAllObjects];
    }
}

@end
