import 'dart:convert';
import 'dart:core';
import 'dart:io';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';
import 'package:yf_ads/bean/yf_ads_draw_native_express_video_bean.dart';
import 'package:yf_ads/callbacks/yf_ads_draw_native_express_video_callback.dart';
import 'package:yf_ads/yf_ads.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/rendering.dart';

class YFAdsDrawNativeExpressVideoView extends StatelessWidget {
  final double? width;
  final double? height;
  final String? positionId;

  ///if you set it true,every ad view will be cached,or especially in scrollview,
  ///such as listview,scroll up or down,every ad view will be requested more than
  ///one time which lead to lost frame,however,ad view to be cached can make it much more better.
  final bool? cache;

  final Function(YFAdsDrawNativeExpressVideoCallBack)? onAdCallBack;

  final int? uniqueId;

  const YFAdsDrawNativeExpressVideoView({
    Key? key,
    this.cache = true,
    this.width = 0,
    this.height = 0,
    this.onAdCallBack,
    this.uniqueId = -1,
    @required this.positionId,
  }) : super(key: key);

  final viewType = "com.yf/draw_native_express_video_platform_view";

  @override
  Widget build(BuildContext context) {
    var reaperDrawNativeExpressVideoBean = YFAdsDrawNativeExpressVideoBean(
        cache: cache,
        width: width,
        height: height,
        uniqueKey: uniqueId,
        //广告位id
        positionId: positionId);

    var createParams = jsonEncode(reaperDrawNativeExpressVideoBean);
    print("createParams is " + createParams);

    Widget androidPlatformView() {
      return PlatformViewLink(
        viewType: viewType,
        surfaceFactory:
            (BuildContext context, PlatformViewController controller) {
          //PlatformViewSurface
          return AndroidViewSurface(
            controller: controller as AndroidViewController,
            gestureRecognizers: const <Factory<OneSequenceGestureRecognizer>>{},
            hitTestBehavior: PlatformViewHitTestBehavior.opaque,
          );
        },
        onCreatePlatformView: (PlatformViewCreationParams params) {
          print("paramsId: ${params.id}");
          return PlatformViewsService.initSurfaceAndroidView(
            id: params.id,
            viewType: viewType,
            layoutDirection: TextDirection.ltr,
            creationParams: createParams,
            creationParamsCodec: const StandardMessageCodec(),
          )
            ..addOnPlatformViewCreatedListener(params.onPlatformViewCreated)
            ..create();
        },
      );
    }

    Widget iosPlatformView() {
      return UiKitView(
          gestureRecognizers: <Factory<OneSequenceGestureRecognizer>>[
            new Factory<OneSequenceGestureRecognizer>(
                () => new EagerGestureRecognizer()),
          ].toSet(),
          layoutDirection: TextDirection.ltr,
          viewType: viewType,
          creationParams: createParams,
          creationParamsCodec: const StandardMessageCodec());
    }

    double adContainerDefaultWidth = MediaQuery.of(context).size.width;
    double adContainerDefaultHeight = MediaQuery.of(context).size.height;
    if (height! > 0) {
      adContainerDefaultHeight = height!;
    }
    if (width! > 0) {
      adContainerDefaultWidth = width!;
    }
    var reaperPlugin = YFAds.instance;
    reaperPlugin.setDrawNativeExpressVideoAdCallBackFunction(onAdCallBack);
    var adFlutterView =
        Platform.isIOS ? iosPlatformView() : androidPlatformView();
    return Container(
      child: adFlutterView,
      width: adContainerDefaultWidth,
      height: adContainerDefaultHeight,
    );
  }
}
