import 'dart:convert';
import 'dart:core';
import 'dart:io';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';
import 'package:yf_ads/bean/yf_ads_express_feed_bean.dart';
import 'package:yf_ads/callbacks/yf_ads_express_feed_callback.dart';
import 'package:yf_ads/yf_ads.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/rendering.dart';

class YFAsdExpressFeedView extends StatelessWidget {
  final String? positionId;
  final Function(YFAdsExpressFeedCallBack)? onAdCallBack;

  final double? width;

  final double? height;

  ///if you want to load several banner ad in list view,
  ///you must set uniqueId parameter with unique value for every item
  final int? uniqueId;

  const YFAsdExpressFeedView({
    Key? key,
    this.onAdCallBack,
    this.uniqueId = -1,
    this.width,
    this.height,
    @required this.positionId,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    final viewType = Platform.isIOS
        ? "com.yf/banner_platform_view"
        : "com.yf/express_feed_platform_view";

    var reaperExpressFeedBean = Platform.isIOS
        ? YFAdsExpressFeedBean(
            width: width,
            height: height,
            uniqueKey: uniqueId,
            expressFeedAd: true,
            //广告位id
            positionId: positionId)
        : YFAdsExpressFeedBean(
            uniqueKey: uniqueId,
            //广告位id
            positionId: positionId);

    var createParams = jsonEncode(reaperExpressFeedBean);
    print("createParams is " + createParams);

    Widget androidPlatformView() {
      return PlatformViewLink(
        viewType: viewType,
        surfaceFactory:
            (BuildContext context, PlatformViewController controller) {
          //PlatformViewSurface
          return AndroidViewSurface(
            controller: controller as AndroidViewController,
            gestureRecognizers: const <Factory<OneSequenceGestureRecognizer>>{},
            hitTestBehavior: PlatformViewHitTestBehavior.opaque,
          );
        },
        onCreatePlatformView: (PlatformViewCreationParams params) {
          print("paramsId: ${params.id}");
          return PlatformViewsService.initSurfaceAndroidView(
            id: params.id,
            viewType: viewType,
            layoutDirection: TextDirection.ltr,
            creationParams: createParams,
            creationParamsCodec: const StandardMessageCodec(),
          )
            ..addOnPlatformViewCreatedListener(params.onPlatformViewCreated)
            ..create();
        },
      );
    }

    Widget iosPlatformView() {
      return UiKitView(
          layoutDirection: TextDirection.ltr,
          viewType: viewType,
          creationParams: createParams,
          creationParamsCodec: const StandardMessageCodec());
    }

    var reaperPlugin = YFAds.instance;
    reaperPlugin.setExpressFeedAdCallBackFunction(onAdCallBack);
    return Platform.isIOS ? iosPlatformView() : androidPlatformView();
  }
}
