import 'dart:convert';
import 'dart:core';
import 'dart:io';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:yf_ads/bean/yf_ads_native_draw_feed_video_bean.dart';
import 'package:yf_ads/callbacks/yf_ads_native_draw_feed_video_callback.dart';
import 'package:yf_ads/yf_ads.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/rendering.dart';

class YFAdsNativeDrawFeedVideoView extends StatelessWidget {
  final double? width;
  final double? height;
  final String? positionId;

  ///if you set it true,every ad view will be cached,or especially in scrollview,
  ///such as listview,scroll up or down,every ad view will be requested more than
  ///one time which lead to lost frame,however,ad view to be cached can make it much more better.
  final bool? cache;

  final Function(YFAdsNativeDrawFeedVideoCallBack)? onAdCallBack;

  final int? uniqueId;

  ///自定义1：设置广告视频区域是否响应点击行为，控制视频暂停、继续播放，默认不响应；
  final bool? interruptVideoPlay;

  ///自定义2：设置视频暂停的图标路径
  final String? pauseIconPath;

  ///自定义2：设置视频暂停的图标工程路径，[pauseIconPath]和[pauseIconResource]设置一个即可
  final int? pauseIconResource;

  ///大小(单位为dp)
  final int? pauseIconSize;

  ///自定义3:布局（icon、标题、描述、按钮）view,适用于Android
  final int? drawFeedItemLayoutId;

  ///自定义4:[drawFeedItemLayoutId]的父布局view，可以在界面设置点赞等按钮,适用于Android
  final int? drawFeedPageLayoutId;

  ///位置，Gravity类值枚举，适用于Android
  final int? gravity;
  final int? topMargin;
  final int? bottomMargin;
  final int? leftMargin;
  final int? rightMargin;

  final Color? backgroundColor;

  const YFAdsNativeDrawFeedVideoView({
    Key? key,
    this.cache = true,
    this.width = 0,
    this.height = 0,
    this.onAdCallBack,
    this.uniqueId = -1,
    @required this.positionId,
    this.interruptVideoPlay,
    this.pauseIconPath,
    this.pauseIconResource,
    this.pauseIconSize,
    this.drawFeedItemLayoutId,
    this.drawFeedPageLayoutId,
    this.gravity,
    this.topMargin,
    this.bottomMargin,
    this.leftMargin,
    this.rightMargin,
    this.backgroundColor = Colors.transparent,
  }) : super(key: key);

  final viewType = "com.yf/native_draw_feed_video_platform_view";

  @override
  Widget build(BuildContext context) {
    var reaperNativeDrawFeedVideoBean = YFAdsNativeDrawFeedVideoBean(
        interruptVideoPlay: interruptVideoPlay,
        pauseIconPath: pauseIconPath,
        pauseIconResource: pauseIconResource,
        pauseIconSize: pauseIconSize,
        drawFeedItemLayoutId: drawFeedItemLayoutId,
        drawFeedPageLayoutId: drawFeedPageLayoutId,
        gravity: gravity,
        topMargin: topMargin,
        bottomMargin: bottomMargin,
        leftMargin: leftMargin,
        rightMargin: rightMargin,
        cache: cache,
        width: width,
        height: height,
        uniqueKey: uniqueId,
        //广告位id
        positionId: positionId!);

    var createParams = jsonEncode(reaperNativeDrawFeedVideoBean);
    print("createParams is " + createParams);

    Widget androidPlatformView() {
      return PlatformViewLink(
        viewType: viewType,
        surfaceFactory:
            (BuildContext context, PlatformViewController controller) {
          //PlatformViewSurface
          return AndroidViewSurface(
            controller: controller as AndroidViewController,
            gestureRecognizers: const <Factory<OneSequenceGestureRecognizer>>{},
            hitTestBehavior: PlatformViewHitTestBehavior.opaque,
          );
        },
        onCreatePlatformView: (PlatformViewCreationParams params) {
          print("paramsId: ${params.id}");
          return PlatformViewsService.initSurfaceAndroidView(
            id: params.id,
            viewType: viewType,
            layoutDirection: TextDirection.ltr,
            creationParams: createParams,
            creationParamsCodec: const StandardMessageCodec(),
          )
            ..addOnPlatformViewCreatedListener(params.onPlatformViewCreated)
            ..create();
        },
      );
    }

    Widget iosPlatformView() {
      return UiKitView(
          gestureRecognizers: <Factory<OneSequenceGestureRecognizer>>[
            new Factory<OneSequenceGestureRecognizer>(
                () => new EagerGestureRecognizer()),
          ].toSet(),
          layoutDirection: TextDirection.ltr,
          viewType: viewType,
          creationParams: createParams,
          creationParamsCodec: const StandardMessageCodec());
    }

    double adContainerDefaultWidth = MediaQuery.of(context).size.width;
    double adContainerDefaultHeight = MediaQuery.of(context).size.height;
    if (height! > 0) {
      adContainerDefaultHeight = height!;
    }
    if (width! > 0) {
      adContainerDefaultWidth = width!;
    }
    var reaperPlugin = YFAds.instance;
    reaperPlugin.setNativeDrawFeedVideoAdCallBackFunction(onAdCallBack);
    var adFlutterView =
        Platform.isIOS ? iosPlatformView() : androidPlatformView();
    return Container(
      color: backgroundColor,
      child: adFlutterView,
      width: adContainerDefaultWidth,
      height: adContainerDefaultHeight,
    );
  }
}
