import 'dart:convert';
import 'dart:core';
import 'dart:io';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';
import 'package:yf_ads/bean/yf_ads_native_android_bean.dart';
import 'package:yf_ads/bean/yf_ads_native_base_bean.dart';
import 'package:yf_ads/bean/yf_ads_native_ios_bean.dart';
import 'package:yf_ads/bean/yf_ads_native_ios_view_frame_bean.dart';
import 'package:yf_ads/yf_ads.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/rendering.dart';

class YFAdsNativeView extends StatelessWidget {
  final YFAdsNativeBaseBean reaperNativeBaseBean;

  const YFAdsNativeView({Key? key, required this.reaperNativeBaseBean})
      : super(key: key);

  final viewType = "com.yf/native_platform_view";
  
  @override
  Widget build(BuildContext context) {
    var createParams = jsonEncode(reaperNativeBaseBean);
    print("createParams is " + createParams);

    Widget androidPlatformView() {
      return PlatformViewLink(
        viewType: viewType,
        surfaceFactory:
            (BuildContext context, PlatformViewController controller) {
          //PlatformViewSurface
          return AndroidViewSurface(
            controller: controller as AndroidViewController,
            gestureRecognizers: const <Factory<OneSequenceGestureRecognizer>>{},
            hitTestBehavior: PlatformViewHitTestBehavior.opaque,
          );
        },
        onCreatePlatformView: (PlatformViewCreationParams params) {
          print("paramsId: ${params.id}");
          return PlatformViewsService.initSurfaceAndroidView(
            id: params.id,
            viewType: viewType,
            layoutDirection: TextDirection.ltr,
            creationParams: createParams,
            creationParamsCodec: const StandardMessageCodec(),
          )
            ..addOnPlatformViewCreatedListener(params.onPlatformViewCreated)
            ..create();
        },
      );
    }

    Widget iosPlatformView() {
      return UiKitView(
          layoutDirection: TextDirection.ltr,
          viewType: viewType,
          creationParams: createParams,
          creationParamsCodec: const StandardMessageCodec());
    }

    double adContainerDefaultWidth = MediaQuery.of(context).size.width;
    double adContainerDefaultHeight = adContainerDefaultWidth * 9 / 16;
    print("adContainerDefaultHeight1 in flutter: $adContainerDefaultHeight");
    if (Platform.isAndroid) {
      YFAdsNativeAndroidBean tempNativeAndroidBean =
          reaperNativeBaseBean as YFAdsNativeAndroidBean;
      int? adContainerHeight = tempNativeAndroidBean.containerHeight;
      if (adContainerHeight != null && adContainerHeight > 0) {
        adContainerDefaultHeight = adContainerHeight.toDouble();
      }
      int? adContainerWidth = tempNativeAndroidBean.containerWidth;
      if (adContainerWidth != null && adContainerWidth > 0) {
        adContainerDefaultWidth = adContainerWidth.toDouble();
      }
    } else if (Platform.isIOS) {
      YFAdsNativeIosBean tempNativeIosBean =
          reaperNativeBaseBean as YFAdsNativeIosBean;
      YFAdsNativeIOSFrameBean? tempNativeIosFrameBean =
          tempNativeIosBean.parentViewFrame;
      double? adContainerHeight = tempNativeIosFrameBean?.height;
      if (adContainerHeight != null && adContainerHeight > 0) {
        adContainerDefaultHeight = adContainerHeight.toDouble();
      }
      double? adContainerWidth = tempNativeIosFrameBean?.width;
      if (adContainerWidth != null && adContainerWidth > 0) {
        adContainerDefaultWidth = adContainerWidth.toDouble();
      }
      double? configMarginTop = tempNativeIosBean.marginTop;
      if (configMarginTop != null && configMarginTop > 0) {
        print(
            "adContainerDefaultHeight2 in flutter: $adContainerDefaultHeight and $configMarginTop");
        adContainerDefaultHeight += configMarginTop;
      }
    }
    print("adContainerDefaultHeight in flutter: $adContainerDefaultHeight");
    var reaperPlugin = YFAds.instance;
    reaperPlugin.setNativeAdCallBackFunction(reaperNativeBaseBean.callBack);
    var adFlutterView =
        Platform.isIOS ? iosPlatformView() : androidPlatformView();
    return Container(
      child: adFlutterView,
      width: adContainerDefaultWidth,
      height: adContainerDefaultHeight,
    );
  }
}
