import 'dart:convert';
import 'dart:core';
import 'dart:io';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';
import 'package:yf_ads/bean/yf_ads_stream_bean.dart';
import 'package:yf_ads/callbacks/yf_ads_callback.dart';
import 'package:yf_ads/yf_ads.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/rendering.dart';

class YFAdsStreamView extends StatelessWidget {
  final String? positionId;

  final Function(YFAdsCallBack)? onAdCallBack;

  ///if you want to load several banner ad in list view,
  ///you must set uniqueId parameter with unique value for every item
  final int? uniqueId;

  final double? width;
  final double? height;

  const YFAdsStreamView({
    Key? key,
    this.onAdCallBack,
    this.uniqueId = -1,
    this.width = 0,
    this.height = 0,
    @required this.positionId,
  }) : super(key: key);

  final viewType = "com.yf/stream_ad_platform_view";

  @override
  Widget build(BuildContext context) {
    var reaperNativeBean = YFAdsStreamBean(
        width: width!.toInt(),
        height: height!.toInt(),
        uniqueKey: uniqueId,
        //广告位id
        positionId: positionId);

    var createParams = jsonEncode(reaperNativeBean);
    print("createParams is " + createParams);

    Widget androidPlatformView() {
      return PlatformViewLink(
        viewType: viewType,
        surfaceFactory:
            (BuildContext context, PlatformViewController controller) {
          //PlatformViewSurface
          return AndroidViewSurface(
            controller: controller as AndroidViewController,
            gestureRecognizers: const <Factory<OneSequenceGestureRecognizer>>{},
            hitTestBehavior: PlatformViewHitTestBehavior.opaque,
          );
        },
        onCreatePlatformView: (PlatformViewCreationParams params) {
          print("paramsId: ${params.id}");
          return PlatformViewsService.initSurfaceAndroidView(
            id: params.id,
            viewType: viewType,
            layoutDirection: TextDirection.ltr,
            creationParams: createParams,
            creationParamsCodec: const StandardMessageCodec(),
          )
            ..addOnPlatformViewCreatedListener(params.onPlatformViewCreated)
            ..create();
        },
      );
    }

    Widget iosPlatformView() {
      return UiKitView(
          gestureRecognizers: <Factory<OneSequenceGestureRecognizer>>[
            new Factory<OneSequenceGestureRecognizer>(
                () => new EagerGestureRecognizer()),
          ].toSet(),
          layoutDirection: TextDirection.ltr,
          viewType: viewType,
          creationParams: createParams,
          creationParamsCodec: const StandardMessageCodec());
    }

    double adContainerDefaultWidth = MediaQuery.of(context).size.width;
    double adContainerDefaultHeight = double.infinity;
    if (height! > 0) {
      adContainerDefaultHeight = height!;
    }
    if (width! > 0) {
      adContainerDefaultWidth = width!;
    }
    var reaperPlugin = YFAds.instance;
    reaperPlugin.setNativeAdCallBackFunction(onAdCallBack);
    var adFlutterView =
        Platform.isIOS ? iosPlatformView() : androidPlatformView();
    return SizedBox(
      width: adContainerDefaultWidth,
      height: adContainerDefaultHeight,
      child: adFlutterView,
    );
  }
}
