import 'dart:ui';

import 'package:yf_ads/sizes/yf_ads_banner_size.dart';

///
/// 自渲染Android banner广告大小枚举
///
enum AndroidBannerAdType {
  AD_SIZE_W690xH388,
  AD_SIZE_W600xH400,
  AD_SIZE_W600xH300,
  AD_SIZE_W600xH260,
  AD_SIZE_W600xH260_L_TEXT_R_PICTURE,
  AD_SIZE_W600xH150,
  AD_SIZE_W600xH150_L_TEXT_R_PICTURE,
  AD_SIZE_W640xH100,
  AD_SIZE_W640xH100_L_TEXT_R_PICTURE,
  AD_SIZE_W600xH90,
  AD_SIZE_W600xH90_L_TEXT_R_PICTURE,
  AD_SIZE_W300xH200,
  AD_SIZE_W300xH200_BLACK,
  AD_SIZE_W600xH540,
  AD_SIZE_W600xH540_TRANSPARENT,
  AD_SIZE_W16H9_B52,
  AD_SIZE_W16H9_B34,
  AD_SIZE_W16H9
}

///
/// 自渲染Android banner广告尺寸处理
///
class AndroidBannerAdSize extends BannerAdSize {
  double getGeneralHeight(int adSizeIndex, double width) {
    double height = 0;
    AndroidBannerAdType adSize = AndroidBannerAdType.values[adSizeIndex];
    switch (adSize) {
      case AndroidBannerAdType.AD_SIZE_W690xH388:
        height = (388 * width) / 690;
        break;
      case AndroidBannerAdType.AD_SIZE_W600xH400:
        height = (400 * width) / 600;
        break;
      case AndroidBannerAdType.AD_SIZE_W600xH300:
        height = (300 * width) / 600;
        break;
      case AndroidBannerAdType.AD_SIZE_W600xH260:
      case AndroidBannerAdType.AD_SIZE_W600xH260_L_TEXT_R_PICTURE:
        height = (260 * width) / 600;
        break;
      case AndroidBannerAdType.AD_SIZE_W600xH150:
      case AndroidBannerAdType.AD_SIZE_W600xH150_L_TEXT_R_PICTURE:
        height = (150 * width) / 600;
        break;
      case AndroidBannerAdType.AD_SIZE_W640xH100:
      case AndroidBannerAdType.AD_SIZE_W640xH100_L_TEXT_R_PICTURE:
        height = (100 * width) / 640;
        break;
      case AndroidBannerAdType.AD_SIZE_W600xH90:
      case AndroidBannerAdType.AD_SIZE_W600xH90_L_TEXT_R_PICTURE:
        height = (90 * width) / 600;
        break;
      case AndroidBannerAdType.AD_SIZE_W300xH200:
      case AndroidBannerAdType.AD_SIZE_W300xH200_BLACK:
        height = (208 * width) / 300;
        break;
      case AndroidBannerAdType.AD_SIZE_W600xH540:
      case AndroidBannerAdType.AD_SIZE_W600xH540_TRANSPARENT:
        height = (540 * width) / 600;
        break;
      case AndroidBannerAdType.AD_SIZE_W16H9_B52:
        height = (16 * width) / 9 + 52;
        break;
      case AndroidBannerAdType.AD_SIZE_W16H9_B34:
        height = (16 * width) / 9 + 34;
        break;
      case AndroidBannerAdType.AD_SIZE_W16H9:
        height = (16 * width) / 9;
        break;
      default:
        height = (100 * width) / 640;
        break;
    }
    return height;
  }

  @override
  double getHeight(int adSizeIndex, double width, double offset, bool aliAd) {
    double height = getGeneralHeight(adSizeIndex, width);
    height += offset;
    return height;
  }

  @override
  double getWidth(int adSizeIndex, double screenWidth, double marginSize) {
    print("getWidth. screenWidthDP: $screenWidth");
    double width = screenWidth - (marginSize * 2);
    return width;
  }

  @override
  Size getExpressBannerAdWidth(
      int adSizeIndex, double screenWidth, double marginSize, bool aqyAd) {
    return const Size(0, 0);
  }
}
