import 'dart:ui';

import 'package:yf_ads/sizes/yf_ads_banner_size.dart';
import 'dart:math';

///
/// 自渲染iOS banner广告大小枚举
///
enum IOSBannerAdType {
  AD_SIZE_W690xH388, //0
  AD_SIZE_W600xH400, //1
  AD_SIZE_W600xH300, //2
  AD_SIZE_W600xH260, //3
  AD_SIZE_W600xH260_L_TEXT_R_PICTURE, //4
  AD_SIZE_W600xH150, //5
  AD_SIZE_W600xH150_L_TEXT_R_PICTURE, //6
  AD_SIZE_W640xH100, //7
  AD_SIZE_W640xH100_L_TEXT_R_PICTURE, //8
  AD_SIZE_W600xH90, //9
  AD_SIZE_W600xH90_L_TEXT_R_PICTURE, //10
  AD_SIZE_W300xH200_BLACK, //11
  AD_SIZE_W300xH200_WHITE, //12
  AD_SIZE_VERTICAL_TITLE_DESC_BELOW_IMAGE, //13
  AD_SIZE_VERTICAL_TITLE_BELOW_IMAGE, //14
  AD_SIZE_VERTICAL_TITLE_ON_BOTTOM, //15
  AD_SIZE_NEW_W600xH150_L_P_R_T, // 16
  AD_SIZE_NEW_W600xH400_T_P_D_T, // 17
  AD_SIZE_NEW_W600xH260_L_P_R_T, // 18
  AD_SIZE_NEW_VER_TD_BP, // 19
  AD_SIZE_NEW_H800, // 20
}

///
/// 自渲染iOS banner广告尺寸处理
///
class IOSBannerAdSize extends BannerAdSize {
  double getRealHeightForWidget(int adSizeIndex, double width) {
    IOSBannerAdType bannerAdType = IOSBannerAdType.values[adSizeIndex];

    double height = 0;
    switch (bannerAdType) {
      case IOSBannerAdType.AD_SIZE_W690xH388:
        height = width * 9 / 16;
        break;
      case IOSBannerAdType.AD_SIZE_W600xH400:
        height = width * 9 / 16 + 40;
        break;
      case IOSBannerAdType.AD_SIZE_W600xH300:
        height = width * 300 / 600;
        break;
      case IOSBannerAdType.AD_SIZE_W600xH260:
      case IOSBannerAdType.AD_SIZE_W600xH260_L_TEXT_R_PICTURE:
        height = width * 260 / 600;
        break;
      case IOSBannerAdType.AD_SIZE_W600xH150:
      case IOSBannerAdType.AD_SIZE_W600xH150_L_TEXT_R_PICTURE:
        height = width * 150 / 600;
        break;
      case IOSBannerAdType.AD_SIZE_W640xH100:
      case IOSBannerAdType.AD_SIZE_W640xH100_L_TEXT_R_PICTURE:
        height = width * 100 / 640;
        break;
      case IOSBannerAdType.AD_SIZE_W600xH90:
      case IOSBannerAdType.AD_SIZE_W600xH90_L_TEXT_R_PICTURE:
        height = width * 90 / 600;
        break;
      case IOSBannerAdType.AD_SIZE_W300xH200_BLACK:
      case IOSBannerAdType.AD_SIZE_W300xH200_WHITE:
        height = width * 9 / 16 + 32;
        break;
      case IOSBannerAdType.AD_SIZE_VERTICAL_TITLE_DESC_BELOW_IMAGE:
        height = width * 16 / 9 + 52;
        break;
      case IOSBannerAdType.AD_SIZE_VERTICAL_TITLE_BELOW_IMAGE:
        height = width * 16 / 9 + 34;
        break;
      case IOSBannerAdType.AD_SIZE_VERTICAL_TITLE_ON_BOTTOM:
        height = width * 16 / 9;
        break;
      case IOSBannerAdType.AD_SIZE_NEW_W600xH150_L_P_R_T:
        width = width >= 220 ? width : 220 ;
        height = width * 150 / 600;
        break;
      case IOSBannerAdType.AD_SIZE_NEW_W600xH400_T_P_D_T:
        width = width >= 220 ? width : 220 ;
        height = width * 16 / 9 + 40;
        break;
      case IOSBannerAdType.AD_SIZE_NEW_W600xH260_L_P_R_T:
        width = width >= 220 ? width : 220 ;
        height = width * 260 / 600;
        break;
      case IOSBannerAdType.AD_SIZE_NEW_VER_TD_BP:
        width = width >= 220 ? width : 220 ;
        height = width * 16 / 9 + 52;
        break;
      case IOSBannerAdType.AD_SIZE_NEW_H800:
        height = 800;
        break;
    }
    return height;
  }

  @override
  double getHeight(int adSizeIndex, double width, double offset, bool aliAd) {
    double bannerHeight = 0;

    if (aliAd) {
      bannerHeight = 80;
    } else {
      bannerHeight = getRealHeightForWidget(adSizeIndex, width);
    }

    return bannerHeight;
  }

  @override
  double getWidth(int adSizeIndex, double screenWidth, double marginSize) {
    int totalSizeTypeNum = IOSBannerAdType.values.length;
    adSizeIndex = adSizeIndex < totalSizeTypeNum ? adSizeIndex : 0;
    double tempMargin = marginSize;
    var wh600X150Index =
        IOSBannerAdType.values.indexOf(IOSBannerAdType.AD_SIZE_W600xH150);
    var wh600X200Index =
        IOSBannerAdType.values.indexOf(IOSBannerAdType.AD_SIZE_W600xH260);
    if (adSizeIndex == wh600X150Index || adSizeIndex == wh600X200Index) {
      double minMargin = (screenWidth - 220) * 0.5;
      tempMargin = min(tempMargin, minMargin);
    }
    double rate = 0.5;
    var whTitleOnBottomVIndex = IOSBannerAdType.values
        .indexOf(IOSBannerAdType.AD_SIZE_VERTICAL_TITLE_ON_BOTTOM);
    var whDescBelowImgVIndex = IOSBannerAdType.values
        .indexOf(IOSBannerAdType.AD_SIZE_VERTICAL_TITLE_DESC_BELOW_IMAGE);
    var whTitleBelowImgVIndex = IOSBannerAdType.values
        .indexOf(IOSBannerAdType.AD_SIZE_VERTICAL_TITLE_BELOW_IMAGE);
    rate = (adSizeIndex == whTitleOnBottomVIndex ||
            adSizeIndex == whTitleBelowImgVIndex ||
            adSizeIndex == whDescBelowImgVIndex)
        ? 0.5
        : 1;

    print("getWidth");

    return screenWidth * rate - tempMargin * 2;
  }

  @override
  Size getExpressBannerAdWidth(
      int adSizeIndex, double screenWidth, double marginSize, bool aqyAd) {
    return const Size(0, 0);
  }
}
