
import 'dart:async';
import 'dart:convert';
import 'dart:io';
import 'package:flutter/services.dart';
import 'package:yf_ads/callbacks/yf_ads_callback.dart';
import 'package:yf_ads/bean/yf_ads_init_sdk_base_bean.dart';
import 'package:yf_ads/bean/yf_ads_interaction_bean.dart';
import 'package:yf_ads/callbacks/yf_ads_draw_native_express_video_callback.dart';
import 'package:yf_ads/callbacks/yf_ads_express_feed_callback.dart';
import 'package:yf_ads/callbacks/yf_ads_native_draw_feed_video_callback.dart';
import 'package:yf_ads/callbacks/yf_ads_reward_video_callback.dart';
import 'package:yf_ads/bean/yf_ads_reward_video_bean.dart';
import 'package:yf_ads/bean/yf_ads_splash_bean.dart';


class YFAds {

  static const MethodChannel _channel = MethodChannel('yf_ads_plugin');
  static const EventChannel _eventChannel =  EventChannel('yf_ads_plugin_event');

  YFAds._privateConstructor() {
    _eventChannel.receiveBroadcastStream().listen(_onEvent, onError: _onError);
  }

  static YFAds get instance => _instance;
  static final YFAds _instance = YFAds._privateConstructor();
  /// for platform call back
  static Function? callBack;
  void _onEvent(Object? event) {
    print("_onEvent is invoke $event");
    if (callBack != null) {
      String reaperAdCallBackStr = event as String;
      var callBackResultMap = jsonDecode(reaperAdCallBackStr);
      YFAdsCallBack reaperAdCallBack;
      if (reaperAdCallBackStr.contains(YFAdsCallBack.YF_ADS_TYPE_REWARD_VIDEO)) {
        reaperAdCallBack =
            YFAdsRewardVideoCallBack.fromJson(callBackResultMap);
      } else if (reaperAdCallBackStr
          .contains(YFAdsCallBack.YF_ADS_TYPE_EXPRESS_FEED)) {
        reaperAdCallBack =
            YFAdsExpressFeedCallBack.fromJson(callBackResultMap);
      } else if (reaperAdCallBackStr
          .contains(YFAdsCallBack.YF_ADS_TYPE_DRAW_NATIVE_EXPRESS_VIDEO)) {
        reaperAdCallBack =
            YFAdsDrawNativeExpressVideoCallBack.fromJson(callBackResultMap);
      } else if (reaperAdCallBackStr
          .contains(YFAdsCallBack.YF_ADS_TYPE_NATIVE_DRAW_FEED_VIDEO)) {
        reaperAdCallBack =
            YFAdsNativeDrawFeedVideoCallBack.fromJson(callBackResultMap);
      } else {
        reaperAdCallBack = YFAdsCallBack.fromJson(callBackResultMap);
      }
      print("callBackResult type is ${reaperAdCallBack.adType}");
      callBack!(reaperAdCallBack);
    }
  }

  void _onError(Object? error) {
    print("_onError is invoke $error");
    if (callBack != null) {
      if (Platform.isIOS) {
        callBack!("$error");
      } else {
        callBack!(error);
      }
    }
  }

  static const KEY_METHOD_INIT_SDK = "initAdsSDK";
  static const KEY_METHOD_REQUEST_REWARD_VIDEO_AD = "requestRewardVideoAd";
  static const KEY_METHOD_REQUEST_INTERACTION_AD = "requestInteractionAd";
  static const KEY_METHOD_SPLASH_AD = "requestSplashAd";
  static const KEY_METHOD_BANNER_AD = "requestBannerAd";
  static const KEY_METHOD_BANNER_PAGE_CLOSED = "notifyBannerPageClosed";
  static const KEY_METHOD_NATIVE_PAGE_CLOSED = "notifyNativeAdPageClosed";
  static const KEY_METHOD_EXPRESSFEED_PAGE_CLOSED =
      "notifyExpressFeedAdPageClosed";
  static const KEY_METHOD_DRAW_NATIVE_EXPRESS_VIDEO_PAGE_CLOSED =
      "notifyDrawNativeExpressVideoAdPageClosed";
  static const KEY_METHOD_NATIVE_DRAW_FEED_VIDEO_AD_PAGE_CLOSED =
      "notifyNativeDrawFeedVideoAdPageClosed";
  static const KEY_METHOD_STREAM_PAGE_CLOSED = "notifyStreamAdPageClosed";

  Future<void> initAdsSDK(
      YFAdsInitSDKBaseBean reaperInitSdkBaseBean) async {
    final String initParameters = jsonEncode(reaperInitSdkBaseBean);
    await _channel.invokeMethod(KEY_METHOD_INIT_SDK, initParameters);
  }

  Future<void> loadRewardVideoAd(
      YFAdsRewardVideoBean reaperRewardVideoBean) async {
    final String rewardVideoParameters = jsonEncode(reaperRewardVideoBean);
    callBack = reaperRewardVideoBean.callBack;
    await _channel.invokeMethod(
        KEY_METHOD_REQUEST_REWARD_VIDEO_AD, rewardVideoParameters);
  }

  Future<void> loadInteractionAd(
      YFAdsInteractionBean reaperInteractionBean) async {
    final String interactionAdParameters = jsonEncode(reaperInteractionBean);
    callBack = reaperInteractionBean.callBack;
    await _channel.invokeMethod(
        KEY_METHOD_REQUEST_INTERACTION_AD, interactionAdParameters);
  }

  Future<void> loadSplashAd(YFAdsSplashBean reaperSplashBean) async {
    callBack = reaperSplashBean.callBack;
    final String splashParameters = jsonEncode(reaperSplashBean);
    await _channel.invokeMethod(KEY_METHOD_SPLASH_AD, splashParameters);
  }


  void setBannerAdCallBackFunction(Function(YFAdsCallBack)? adCallBack) {
    callBack = adCallBack;
  }


  void setSplashAdCallBackFunction(Function(YFAdsCallBack)? adCallBack) {
    callBack = adCallBack;
  }

  void setNativeAdCallBackFunction(Function(YFAdsCallBack)? adCallBack) {
    callBack = adCallBack;
  }

  void setExpressFeedAdCallBackFunction(
      Function(YFAdsExpressFeedCallBack)? adCallBack) {
    callBack = adCallBack;
  }

  void setDrawNativeExpressVideoAdCallBackFunction(
      Function(YFAdsDrawNativeExpressVideoCallBack)? adCallBack) {
    callBack = adCallBack;
  }

  void setNativeDrawFeedVideoAdCallBackFunction(
      Function(YFAdsNativeDrawFeedVideoCallBack)? adCallBack) {
    callBack = adCallBack;
  }

  Future<void> clearBannerPageCache() async {
    await _channel.invokeMethod(KEY_METHOD_BANNER_PAGE_CLOSED);
  }

  Future<void> clearNativePageCache() async {
    await _channel.invokeMethod(KEY_METHOD_NATIVE_PAGE_CLOSED);
  }

  Future<void> clearExpressFeedPageCache() async {
    if (Platform.isIOS) {
      await _channel.invokeMethod(KEY_METHOD_BANNER_PAGE_CLOSED);
    } else {
      await _channel.invokeMethod(KEY_METHOD_EXPRESSFEED_PAGE_CLOSED);
    }
  }

  Future<void> clearDrawNativeExpressVideoPageCache() async {
    await _channel
        .invokeMethod(KEY_METHOD_DRAW_NATIVE_EXPRESS_VIDEO_PAGE_CLOSED);
  }

  Future<void> clearNativeDrawFeedVideoPageCache() async {
    await _channel
        .invokeMethod(KEY_METHOD_NATIVE_DRAW_FEED_VIDEO_AD_PAGE_CLOSED);
  }

  Future<void> clearStreamPageCache() async {
    await _channel.invokeMethod(KEY_METHOD_STREAM_PAGE_CLOSED);
  }
}
